@@ -1,5 +1,5 @@
 /*
- * $Id: acconfig.h,v 1.79 2001/07/13 18:21:30 grubba Exp $
+ * $Id: acconfig.h,v 1.81 2003/12/13 20:37:53 jonasw Exp $
  */
 #ifndef MACHINE_H
 #define MACHINE_H
@@ -308,6 +308,12 @@
 /* Define this if your pthreads have pthread_condattr_default */
 #undef HAVE_PTHREAD_CONDATTR_DEFAULT
 
+/* Define if your pthreads have PTHREAD_MUTEX_ERRORCHECK */
+#undef HAVE_PTHREAD_MUTEX_ERRORCHECK
+
+/* Define if your pthreads have PTHREAD_MUTEX_ERRORCHECK_NP */
+#undef HAVE_PTHREAD_MUTEX_ERRORCHECK_NP
+
 /* Define if your pthreads have PTHREAD_MUTEX_RECURSIVE */
 #undef HAVE_PTHREAD_MUTEX_RECURSIVE
 
@@ -329,6 +335,9 @@
 /* Define if you have the pthread_yield function.  */
 #undef HAVE_PTHREAD_YIELD
 
+/* Define if you have the pthread_yield_np function.  */
+#undef HAVE_PTHREAD_YIELD_NP
+
 /* Hack for stupid glibc linuxthreads */
 #undef HAVE_PTHREAD_INITIAL_THREAD_BOS
 
