@@ -15,6 +15,7 @@
 #include "opcodes.h"
 #include "pike_error.h"
 #include "pike_types.h"
+#include "cyclic.h"
 #include "fsort.h"
 #include "builtin_functions.h"
 #include "pike_memory.h"
@@ -24,7 +25,7 @@
 #include "stuff.h"
 #include "bignum.h"
 
-RCSID("$Id: array.c,v 1.106 2002/03/06 11:01:58 grubba Exp $");
+RCSID("$Id: array.c,v 1.110 2003/11/09 02:40:35 mast Exp $");
 
 PMOD_EXPORT struct array empty_array=
 {
@@ -958,7 +959,7 @@ INT32 set_lookup(struct array *a, struct svalue *s)
 #endif
 
   /* objects may have `< `> operators, evil stuff! */
-  if(s->type != T_OBJECT && !(a->flags & BIT_OBJECT))
+  if(s->type != T_OBJECT && !(a->type_field & BIT_OBJECT))
   {
     /* face it, it's not there */
     if( (((2 << s->type) -1) & a->type_field) == 0)
@@ -979,7 +980,7 @@ INT32 switch_lookup(struct array *a, struct svalue *s)
   if(d_flag > 1)  array_check_type_field(a);
 #endif
   /* objects may have `< `> operators, evil stuff! */
-  if(s->type != T_OBJECT && !(a->flags & BIT_OBJECT))
+  if(s->type != T_OBJECT && !(a->type_field & BIT_OBJECT))
   {
     if( (((2 << s->type) -1) & a->type_field) == 0)
       return -1;
@@ -1132,7 +1133,8 @@ INT32 * merge(struct array *a,struct array *b,INT32 opcode)
     array_check_type_field(b);
   }
 #endif
-  if(!(a->type_field & b->type_field))
+  if(!(a->type_field & b->type_field) &&
+     !((a->type_field | b->type_field) & BIT_OBJECT))
   {
     /* do smart optimizations */
     switch(opcode)
@@ -1518,7 +1520,8 @@ PMOD_EXPORT struct array *subtract_arrays(struct array *a, struct array *b)
 #endif
   check_array_for_destruct(a);
 
-  if(a->type_field & b->type_field)
+  if((a->type_field & b->type_field) ||
+     ((a->type_field | b->type_field) & BIT_OBJECT))
   {
     return merge_array_with_order(a, b, PIKE_ARRAY_OP_SUB);
   }else{
@@ -1542,7 +1545,8 @@ PMOD_EXPORT struct array *and_arrays(struct array *a, struct array *b)
 #endif
   check_array_for_destruct(a);
 
-  if(a->type_field & b->type_field)
+  if((a->type_field & b->type_field) ||
+     ((a->type_field | b->type_field) & BIT_OBJECT))
   {
     return merge_array_with_order(a, b, PIKE_ARRAY_OP_AND_LEFT);
   }else{
@@ -1895,22 +1899,37 @@ PMOD_EXPORT void apply_array(struct array *a, INT32 args)
 {
   INT32 e;
   struct array *ret;
-  ptrdiff_t argp;
+  struct svalue *argp = Pike_sp - args;
+  TYPE_FIELD new_types = 0;
+  struct array *cycl;
+  DECLARE_CYCLIC();
 
-  argp=Pike_sp-args - Pike_interpreter.evaluator_stack;
-
-  check_stack(a->size + args + 1);
   check_array_for_destruct(a);
-  for(e=0;e<a->size;e++)
-  {
-    assign_svalues_no_free(Pike_sp, Pike_interpreter.evaluator_stack + argp,
-			   args, BIT_MIXED);
-    Pike_sp+=args;
-    apply_svalue(ITEM(a)+e,args);
+  check_stack(120 + args + 1);
+
+  /* FIXME: Ought to use a better key on the arguments below. */
+  if (!(cycl = (struct array *)BEGIN_CYCLIC(a, args))) {
+    BEGIN_AGGREGATE_ARRAY(a->size) {
+      SET_CYCLIC_RET(Pike_sp[-1].u.array);
+      for (e=0;e<a->size;e++) {
+	assign_svalues_no_free(Pike_sp, argp, args, BIT_MIXED);
+	Pike_sp+=args;
+	apply_svalue(ITEM(a)+e,args);
+	new_types |= 1 << Pike_sp[-1].type;
+	DO_AGGREGATE_ARRAY(120);
+      }
+    }
+    END_AGGREGATE_ARRAY;
+
+    Pike_sp[-1].u.array->type_field = new_types;
+#ifdef PIKE_DEBUG
+    array_check_type_field(Pike_sp[-1].u.array);
+#endif
+  } else {
+    ref_push_array(cycl);
   }
-  ret=aggregate_array(a->size);
-  pop_n_elems(args);
-  push_array(ret);
+  END_CYCLIC();
+  stack_pop_n_elems_keep_top(args);
 }
 
 PMOD_EXPORT struct array *reverse_array(struct array *a)
@@ -2037,14 +2056,14 @@ void gc_mark_array_as_referenced(struct array *a)
 #endif
 	  t = 0;
 	  for(e=0;e<a->size;e++)
-	    if (!gc_mark_weak_svalues(a->item+e, 1)) {
+	    if (!debug_gc_mark_weak_svalues(a->item+e, 1, T_ARRAY, a)) {
 	      a->item[d++]=a->item[e];
 	      t |= 1 << a->item[e].type;
 	    }
 	  a->size=d;
 	}
 	else
-	  if (!(t = gc_mark_weak_svalues(a->item, a->size)))
+	  if (!(t = debug_gc_mark_weak_svalues(a->item, a->size, T_ARRAY, a)))
 	    t = a->type_field;
 
 	/* Ugly, but we are not allowed to change type_field
@@ -2060,7 +2079,7 @@ void gc_mark_array_as_referenced(struct array *a)
       }
       else {
 	TYPE_FIELD t;
-	if ((t = gc_mark_svalues(ITEM(a), a->size))) {
+	if ((t = debug_gc_mark_svalues(ITEM(a), a->size, T_ARRAY, a))) {
 	  if(!(a->type_field & BIT_UNFINISHED) || a->refs!=1)
 	    a->type_field = t;
 	  else
@@ -2104,7 +2123,6 @@ void real_gc_cycle_check_array(struct array *a, int weak)
   } GC_CYCLE_LEAVE;
 }
 
-#ifdef PIKE_DEBUG
 unsigned gc_touch_all_arrays(void)
 {
   unsigned n = 0;
@@ -2118,7 +2136,6 @@ unsigned gc_touch_all_arrays(void)
   } while (a != &empty_array);
   return n;
 }
-#endif
 
 void gc_check_all_arrays(void)
 {
@@ -2169,7 +2186,7 @@ void gc_zap_ext_weak_refs_in_arrays(void)
     gc_mark_array_pos = a->next;
     gc_mark_array_as_referenced(a);
   }
-  discard_queue(&gc_mark_queue);
+  gc_mark_discard_queue();
 }
 
 void gc_free_all_unreferenced_arrays(void)
