@@ -5,7 +5,7 @@
 \*/
 /**/
 #include "global.h"
-RCSID("$Id: backend.c,v 1.65 2001/07/02 20:32:54 mast Exp $");
+RCSID("$Id: backend.c,v 1.67 2003/08/04 14:56:34 mast Exp $");
 #include "fdlib.h"
 #include "backend.h"
 #include <errno.h>
@@ -67,6 +67,7 @@ PMOD_EXPORT int fds_size = 0;
 
 void grow_fds(void)
 {
+  int old_size = fds_size;
   if( !fds_size )
     fds_size = 16;
   fds_size *= 2;
@@ -74,7 +75,7 @@ void grow_fds(void)
   if( !fds )
     fatal("Out of memory in backend::grow_fds()\n"
           "Tried to allocate %d fd_datum structs\n", fds_size);
-  MEMSET( fds+(fds_size/2), 0, fds_size*sizeof(struct fd_datum)/2 );
+  MEMSET(fds + old_size, 0, (fds_size - old_size) * sizeof(struct fd_datum));
 }
 
 
@@ -874,6 +875,8 @@ void backend(void)
 	  }
 #endif /* PIKE_DEBUG */
 	  IF_PD(fprintf(stderr, "BACKEND: POLLHUP | POLLERR\n"));
+	  /* We don't want to keep this fd anymore. */
+	  POLL_FD_CLR(fd, ~0);
 	  if (fds[fd].read.callback) {
 	    IF_PD(fprintf(stderr, "BACKEND: read_callback(%d, %p)\n",
 			  fd, fds[fd].read.data));
@@ -883,8 +886,6 @@ void backend(void)
 			  fd, fds[fd].write.data));
 	    (*(fds[fd].write.callback))(fd, fds[fd].write.data);
 	  }
-	  /* We don't want to keep this fd anymore. */
-	  POLL_FD_CLR(fd, ~0);
 #ifdef PIKE_DEBUG
 	  handled = 1;
 #endif /* PIKE_DEBUG */
