@@ -1,9 +1,9 @@
 #include "global.h"
 #include "fdlib.h"
-#include "error.h"
+#include "pike_error.h"
 #include <math.h>
 
-RCSID("$Id: fdlib.c,v 1.23 1999/01/01 00:52:01 hubbe Exp $");
+RCSID("$Id: fdlib.c,v 1.54 2003/09/16 16:45:30 mast Exp $");
 
 #ifdef HAVE_WINSOCK_H
 
@@ -13,13 +13,19 @@ RCSID("$Id: fdlib.c,v 1.23 1999/01/01 00:52:01 hubbe Exp $");
 static MUTEX_T fd_mutex;
 #endif
 
-long da_handle[MAX_OPEN_FILEDESCRIPTORS];
+HANDLE da_handle[MAX_OPEN_FILEDESCRIPTORS];
 int fd_type[MAX_OPEN_FILEDESCRIPTORS];
 int first_free_handle;
 
-#define FDDEBUG(X)
+/* #define FD_DEBUG */
 
-char *fd_info(int fd)
+#ifdef FD_DEBUG
+#define FDDEBUG(X) X
+#else
+#define FDDEBUG(X)
+#endif
+
+PMOD_EXPORT char *debug_fd_info(int fd)
 {
   if(fd<0)
     return "BAD";
@@ -37,7 +43,7 @@ char *fd_info(int fd)
   }
 }
 
-int fd_query_properties(int fd, int guess)
+PMOD_EXPORT int debug_fd_query_properties(int fd, int guess)
 {
   switch(fd_type[fd])
   {
@@ -66,12 +72,12 @@ void fd_init()
   }
   FDDEBUG(fprintf(stderr,"Using %s\n",wsadata.szDescription));
   
-  fd_type[0]=FD_CONSOLE;
-  da_handle[0]=(long)GetStdHandle(STD_INPUT_HANDLE);
-  fd_type[1]=FD_CONSOLE;
-  da_handle[1]=(long)GetStdHandle(STD_OUTPUT_HANDLE);
-  fd_type[2]=FD_CONSOLE;
-  da_handle[2]=(long)GetStdHandle(STD_ERROR_HANDLE);
+  fd_type[0] = FD_CONSOLE;
+  da_handle[0] = GetStdHandle(STD_INPUT_HANDLE);
+  fd_type[1] = FD_CONSOLE;
+  da_handle[1] = GetStdHandle(STD_OUTPUT_HANDLE);
+  fd_type[2] = FD_CONSOLE;
+  da_handle[2] = GetStdHandle(STD_ERROR_HANDLE);
 
   first_free_handle=3;
   for(e=3;e<MAX_OPEN_FILEDESCRIPTORS-1;e++)
@@ -86,11 +92,232 @@ void fd_exit()
   mt_destroy(&fd_mutex);
 }
 
-FD fd_open(char *file, int open_mode, int create_mode)
+
+#define ISSEPARATOR(a)  ((a) == '\\' || (a) == '/')
+
+/*
+ * IsUncRoot - returns TRUE if the argument is a UNC name specifying a
+ *             root share.  That is, if it is of the form
+ *             \\server\share\.  This routine will also return true if
+ *             the argument is of the form \\server\share (no trailing
+ *             slash).
+ *
+ *             Forward slashes ('/') may be used instead of
+ *             backslashes ('\').
+ */
+
+static int IsUncRoot(char *path)
+{
+  /* root UNC names start with 2 slashes */
+  if ( strlen(path) >= 5 &&     /* minimum string is "//x/y" */
+       ISSEPARATOR(path[0]) &&
+       ISSEPARATOR(path[1]) )
+  {
+    char * p = path + 2 ;
+    
+    /* find the slash between the server name and share name */
+    while ( *++p )
+      if ( ISSEPARATOR(*p) )
+        break ;
+    
+    if ( *p && p[1] )
+    {
+      /* is there a further slash? */
+      while ( *++p )
+        if ( ISSEPARATOR(*p) )
+          break ;
+      
+      /* final slash (if any) */
+      if ( !*p || !p[1])
+        return 1;
+    }
+  }
+  
+  return 0 ;
+}
+
+static int low_stat (char *file, struct stat *buf)
+{
+  char            *path;
+  int              drive;       /* A: = 1, B: = 2, ... */
+  char             pathbuf[ _MAX_PATH ];
+  HANDLE           hFind;
+  WIN32_FIND_DATA  findbuf;
+
+  /* don't allow wildcards */
+  if (strpbrk(file, "?*"))
+  {
+    errno = ENOENT;
+    return(-1);
+  }
+  
+  /* get disk from file */
+  if (file[1] == ':')
+  {
+    if ( *file && !file[2] )
+    {
+      /* return an error if file is just drive letter and colon */
+      errno = ENOENT;           
+      return( -1 );
+    }
+    drive = toupper(*file) - 'A' + 1;
+  }
+  else
+    drive = _getdrive();
+  
+  /* get info for file */
+  hFind = FindFirstFile(file, &findbuf);
+  if ( hFind == INVALID_HANDLE_VALUE )
+  {
+    
+    if ( !(strpbrk(file, "./\\") &&
+           (path = _fullpath( pathbuf, file, _MAX_PATH )) &&
+           /* root dir. ('C:\') or UNC root dir. ('\\server\share\') */
+           ((strlen( path ) == 3) || IsUncRoot(path)) &&
+           (GetDriveType( path ) > 1) ) )
+    {
+      errno = ENOENT;
+      return( -1 );
+    }
+    
+    /* Root directories ( C:\ and \\server\share\) are faked */
+    findbuf.dwFileAttributes = FILE_ATTRIBUTE_DIRECTORY;
+    findbuf.nFileSizeHigh = 0;
+    findbuf.nFileSizeLow = 0;
+    findbuf.cFileName[0] = '\0';
+    
+    buf->st_mtime = __loctotime_t(1980,1,1,0,0,0, -1);
+    buf->st_atime = buf->st_mtime;
+    buf->st_ctime = buf->st_mtime;
+  }
+  else
+  {
+    SYSTEMTIME SystemTime;
+    SYSTEMTIME UtcSTime;
+    
+    if ( !FileTimeToSystemTime( &findbuf.ftLastWriteTime,
+                                &UtcSTime )                    ||
+         !SystemTimeToTzSpecificLocalTime(NULL, &UtcSTime,
+                                          &SystemTime) )
+    {
+      _dosmaperr( GetLastError() );
+      FindClose( hFind );
+      return( -1 );
+    }
+
+    buf->st_mtime = __loctotime_t( SystemTime.wYear,
+                                   SystemTime.wMonth,
+                                   SystemTime.wDay,
+                                   SystemTime.wHour,
+                                   SystemTime.wMinute,
+                                   SystemTime.wSecond,
+                                   -1 );
+    
+    if ( findbuf.ftLastAccessTime.dwLowDateTime ||
+         findbuf.ftLastAccessTime.dwHighDateTime )
+    {
+      if ( !FileTimeToSystemTime( &findbuf.ftLastAccessTime,
+                                  &UtcSTime )                    ||
+           !SystemTimeToTzSpecificLocalTime(NULL, &UtcSTime,
+                                            &SystemTime) )
+      {
+        _dosmaperr( GetLastError() );
+        FindClose( hFind );
+        return( -1 );
+      }
+      
+      buf->st_atime = __loctotime_t( SystemTime.wYear,
+                                     SystemTime.wMonth,
+                                     SystemTime.wDay,
+                                     SystemTime.wHour,
+                                     SystemTime.wMinute,
+                                     SystemTime.wSecond,
+                                     -1 );
+    } else
+      buf->st_atime = buf->st_mtime ;
+    
+    if ( findbuf.ftCreationTime.dwLowDateTime ||
+         findbuf.ftCreationTime.dwHighDateTime )
+    {
+      if ( !FileTimeToSystemTime( &findbuf.ftCreationTime,
+                                  &UtcSTime )                    ||
+           !SystemTimeToTzSpecificLocalTime(NULL, &UtcSTime,
+                                            &SystemTime) )
+      {
+        _dosmaperr( GetLastError() );
+        FindClose( hFind );
+        return( -1 );
+      }
+      
+      buf->st_ctime = __loctotime_t( SystemTime.wYear,
+                                     SystemTime.wMonth,
+                                     SystemTime.wDay,
+                                     SystemTime.wHour,
+                                     SystemTime.wMinute,
+                                     SystemTime.wSecond,
+                                     -1 );
+    } else
+      buf->st_ctime = buf->st_mtime ;
+    
+    FindClose(hFind);
+  }
+  
+  buf->st_mode = __dtoxmode(findbuf.dwFileAttributes, file);
+  buf->st_nlink = 1;
+  buf->st_size = findbuf.nFileSizeLow;
+  
+  buf->st_uid = buf->st_gid = buf->st_ino = 0; /* unused entries */
+  buf->st_rdev = buf->st_dev = (_dev_t)(drive - 1); /* A=0, B=1, ... */
+  
+  return(0);
+}
+
+int debug_fd_stat(char *file, struct stat *buf)
+{
+  ptrdiff_t l = strlen(file);
+  char fname[MAX_PATH];
+
+  if(file[l-1]=='/' || file[l-1]=='\\')
+  {
+    do l--;
+    while(l && ( file[l]=='/' || file[l]=='\\' ));
+    l++;
+    if(l+1 > sizeof(fname))
+    {
+      errno=EINVAL;
+      return -1;
+    }
+    MEMCPY(fname, file, l);
+    fname[l]=0;
+    file=fname;
+  }
+  return low_stat(file, buf);
+}
+
+PMOD_EXPORT FD debug_fd_open(char *file, int open_mode, int create_mode)
 {
   HANDLE x;
   FD fd;
   DWORD omode,cmode,amode;
+
+  ptrdiff_t l = strlen(file);
+  char fname[MAX_PATH];
+
+  if(file[l-1]=='/' || file[l-1]=='\\')
+  {
+    do l--;
+    while(l && ( file[l]=='/' || file[l]=='\\' ));
+    l++;
+    if(l+1 > sizeof(fname))
+    {
+      errno=EINVAL;
+      return -1;
+    }
+    MEMCPY(fname, file, l);
+    fname[l]=0;
+    file=fname;
+  }
+
   omode=0;
   FDDEBUG(fprintf(stderr,"fd_open(%s,0x%x,%o)\n",file,open_mode,create_mode));
   if(first_free_handle == FD_NO_MORE_FREE)
@@ -127,7 +354,7 @@ FD fd_open(char *file, int open_mode, int create_mode)
       break;
   }
 
-  if(create_mode & 4)
+  if(create_mode & 0222)
   {
     amode=FILE_ATTRIBUTE_NORMAL;
   }else{
@@ -143,7 +370,7 @@ FD fd_open(char *file, int open_mode, int create_mode)
 	       NULL);
 
   
-  if(x==INVALID_HANDLE_VALUE)
+  if(x == DO_NOT_WARN(INVALID_HANDLE_VALUE))
   {
     errno=GetLastError();
     return -1;
@@ -156,7 +383,7 @@ FD fd_open(char *file, int open_mode, int create_mode)
   fd=first_free_handle;
   first_free_handle=fd_type[fd];
   fd_type[fd]=FD_FILE;
-  da_handle[fd]=(long)x;
+  da_handle[fd] = x;
 
   mt_unlock(&fd_mutex);
 
@@ -168,7 +395,7 @@ FD fd_open(char *file, int open_mode, int create_mode)
   return fd;
 }
 
-FD fd_socket(int domain, int type, int proto)
+PMOD_EXPORT FD debug_fd_socket(int domain, int type, int proto)
 {
   FD fd;
   SOCKET s;
@@ -189,12 +416,13 @@ FD fd_socket(int domain, int type, int proto)
     return -1;
   }
   
-  SetHandleInformation(s,HANDLE_FLAG_INHERIT|HANDLE_FLAG_PROTECT_FROM_CLOSE,0);
+  SetHandleInformation((HANDLE)s,
+		       HANDLE_FLAG_INHERIT|HANDLE_FLAG_PROTECT_FROM_CLOSE, 0);
   mt_lock(&fd_mutex);
   fd=first_free_handle;
   first_free_handle=fd_type[fd];
-  fd_type[fd]=FD_SOCKET;
-  da_handle[fd]=(long)s;
+  fd_type[fd] = FD_SOCKET;
+  da_handle[fd] = (HANDLE)s;
   mt_unlock(&fd_mutex);
 
   FDDEBUG(fprintf(stderr,"New socket: %d (%d)\n",fd,s));
@@ -202,7 +430,7 @@ FD fd_socket(int domain, int type, int proto)
   return fd;
 }
 
-int fd_pipe(int fds[2])
+PMOD_EXPORT int debug_fd_pipe(int fds[2] DMALLOC_LINE_ARGS)
 {
   HANDLE files[2];
   mt_lock(&fd_mutex);
@@ -218,7 +446,7 @@ int fd_pipe(int fds[2])
     errno=GetLastError();
     return -1;
   }
-
+  
   FDDEBUG(fprintf(stderr,"ReadHANDLE=%d WriteHANDLE=%d\n",files[0],files[1]));
   
   SetHandleInformation(files[0],HANDLE_FLAG_INHERIT|HANDLE_FLAG_PROTECT_FROM_CLOSE,0);
@@ -227,25 +455,32 @@ int fd_pipe(int fds[2])
   fds[0]=first_free_handle;
   first_free_handle=fd_type[fds[0]];
   fd_type[fds[0]]=FD_PIPE;
-  da_handle[fds[0]]=(long)files[0];
+  da_handle[fds[0]] = files[0];
 
   fds[1]=first_free_handle;
   first_free_handle=fd_type[fds[1]];
   fd_type[fds[1]]=FD_PIPE;
-  da_handle[fds[1]]=(long)files[1];
+  da_handle[fds[1]] = files[1];
 
   mt_unlock(&fd_mutex);
   FDDEBUG(fprintf(stderr,"New pipe: %d (%d) -> %d (%d)\n",fds[0],files[0], fds[1], fds[1]));;
 
+#ifdef DEBUG_MALLOC
+  debug_malloc_register_fd( fds[0], DMALLOC_LOCATION());
+  debug_malloc_register_fd( fds[1], DMALLOC_LOCATION());
+#endif
+  
   return 0;
 }
 
-FD fd_accept(FD fd, struct sockaddr *addr, int *addrlen)
+PMOD_EXPORT FD debug_fd_accept(FD fd, struct sockaddr *addr,
+			       ACCEPT_SIZE_T *addrlen)
 {
   FD new_fd;
   SOCKET s;
   mt_lock(&fd_mutex);
-  FDDEBUG(fprintf(stderr,"Accept on %d (%d)..\n",fd,da_handle[fd]));
+  FDDEBUG(fprintf(stderr,"Accept on %d (%ld)..\n",
+		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd])));
   if(first_free_handle == FD_NO_MORE_FREE)
   {
     mt_unlock(&fd_mutex);
@@ -268,14 +503,17 @@ FD fd_accept(FD fd, struct sockaddr *addr, int *addrlen)
     return -1;
   }
   
-  SetHandleInformation(s,HANDLE_FLAG_INHERIT|HANDLE_FLAG_PROTECT_FROM_CLOSE,0);
+  SetHandleInformation((HANDLE)s,
+		       HANDLE_FLAG_INHERIT|HANDLE_FLAG_PROTECT_FROM_CLOSE, 0);
   mt_lock(&fd_mutex);
   new_fd=first_free_handle;
   first_free_handle=fd_type[new_fd];
   fd_type[new_fd]=FD_SOCKET;
-  da_handle[new_fd]=(long)s;
+  da_handle[new_fd] = (HANDLE)s;
 
-  FDDEBUG(fprintf(stderr,"Accept on %d (%d) returned new socket: %d (%d)\n",fd,da_handle[fd],new_fd,s));
+  FDDEBUG(fprintf(stderr,"Accept on %d (%ld) returned new socket: %d (%ld)\n",
+		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd]),
+		  new_fd, PTRDIFF_T_TO_LONG((ptrdiff_t)s)));
 
   mt_unlock(&fd_mutex);
 
@@ -284,44 +522,62 @@ FD fd_accept(FD fd, struct sockaddr *addr, int *addrlen)
 
 
 #define SOCKFUN(NAME,X1,X2) \
-int PIKE_CONCAT(fd_,NAME) X1 { SOCKET ret; \
-  FDDEBUG(fprintf(stderr, #NAME " on %d (%d)\n",fd,da_handle[fd])); \
+PMOD_EXPORT int PIKE_CONCAT(debug_fd_,NAME) X1 { \
+  SOCKET s; \
+  int ret; \
+  FDDEBUG(fprintf(stderr, #NAME " on %d (%ld)\n", \
+		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd]))); \
   mt_lock(&fd_mutex); \
   if(fd_type[fd] != FD_SOCKET) { \
      mt_unlock(&fd_mutex); \
-     errno=ENOTSUPP; \
+     errno = ENOTSUPP; \
      return -1; \
    } \
-  ret=(SOCKET)da_handle[fd]; \
+  s = (SOCKET)da_handle[fd]; \
   mt_unlock(&fd_mutex); \
-   ret=NAME X2; \
-   if(ret == SOCKET_ERROR) errno=WSAGetLastError(); \
-   FDDEBUG(fprintf(stderr, #NAME " returned %d (%d)\n",ret,errno)); \
-   return (int)ret; \
+  ret = NAME X2; \
+  if(ret == SOCKET_ERROR) { \
+    errno = WSAGetLastError(); \
+    ret = -1; \
+  } \
+  FDDEBUG(fprintf(stderr, #NAME " returned %d (%d)\n", ret, errno)); \
+  return ret; \
 }
 
 #define SOCKFUN1(NAME,T1) \
-   SOCKFUN(NAME, (FD fd, T1 a), (ret, a) )
+   SOCKFUN(NAME, (FD fd, T1 a), (s, a) )
 
 #define SOCKFUN2(NAME,T1,T2) \
-   SOCKFUN(NAME, (FD fd, T1 a, T2 b), (ret, a, b) )
+   SOCKFUN(NAME, (FD fd, T1 a, T2 b), (s, a, b) )
 
 #define SOCKFUN3(NAME,T1,T2,T3) \
-   SOCKFUN(NAME, (FD fd, T1 a, T2 b, T3 c), (ret, a, b, c) )
+   SOCKFUN(NAME, (FD fd, T1 a, T2 b, T3 c), (s, a, b, c) )
 
 #define SOCKFUN4(NAME,T1,T2,T3,T4) \
-   SOCKFUN(NAME, (FD fd,T1 a,T2 b,T3 c,T4 d), (ret,a,b,c,d) )
+   SOCKFUN(NAME, (FD fd, T1 a, T2 b, T3 c, T4 d), (s, a, b, c, d) )
 
 #define SOCKFUN5(NAME,T1,T2,T3,T4,T5) \
-   SOCKFUN(NAME, (FD fd,T1 a,T2 b,T3 c,T4 d,T5 e), (ret,a,b,c,d,e))
+   SOCKFUN(NAME, (FD fd, T1 a, T2 b, T3 c, T4 d, T5 e), (s, a, b, c, d, e))
 
 
 SOCKFUN2(bind, struct sockaddr *, int)
-int fd_connect (FD fd, struct sockaddr *a, int len)
+SOCKFUN4(getsockopt,int,int,void*,ACCEPT_SIZE_T *)
+SOCKFUN4(setsockopt,int,int,void*,int)
+SOCKFUN3(recv,void *,int,int)
+SOCKFUN2(getsockname,struct sockaddr *,ACCEPT_SIZE_T *)
+SOCKFUN2(getpeername,struct sockaddr *,ACCEPT_SIZE_T *)
+SOCKFUN5(recvfrom,void *,int,int,struct sockaddr *,ACCEPT_SIZE_T *)
+SOCKFUN3(send,void *,int,int)
+SOCKFUN5(sendto,void *,int,int,struct sockaddr *,unsigned int)
+SOCKFUN1(shutdown, int)
+SOCKFUN1(listen, int)
+
+PMOD_EXPORT int debug_fd_connect (FD fd, struct sockaddr *a, int len)
 {
   SOCKET ret;
   mt_lock(&fd_mutex);
-  FDDEBUG(fprintf(stderr, "connect on %d (%d)\n",fd,da_handle[fd]);
+  FDDEBUG(fprintf(stderr, "connect on %d (%ld)\n",
+		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd]));
 	  for(ret=0;ret<len;ret++)
 	  fprintf(stderr," %02x",((unsigned char *)a)[ret]);
 	  fprintf(stderr,"\n");
@@ -337,26 +593,17 @@ int fd_connect (FD fd, struct sockaddr *a, int len)
   ret=connect(ret,a,len); 
   if(ret == SOCKET_ERROR) errno=WSAGetLastError(); 
   FDDEBUG(fprintf(stderr, "connect returned %d (%d)\n",ret,errno)); 
-  return (int)ret; 
+  return DO_NOT_WARN((int)ret);
 }
-SOCKFUN4(getsockopt,int,int,void*,int*)
-SOCKFUN4(setsockopt,int,int,void*,int)
-SOCKFUN3(recv,void *,int,int)
-SOCKFUN2(getsockname,struct sockaddr *,int)
-SOCKFUN2(getpeername,struct sockaddr *,int)
-SOCKFUN5(recvfrom,void *,int,int,struct sockaddr *,int*)
-SOCKFUN3(send,void *,int,int)
-SOCKFUN5(sendto,void *,int,int,struct sockaddr *,int*)
-SOCKFUN1(shutdown, int)
-SOCKFUN1(listen, int)
 
-int fd_close(FD fd)
+PMOD_EXPORT int debug_fd_close(FD fd)
 {
-  long h;
+  HANDLE h;
   int type;
   mt_lock(&fd_mutex);
-  h=(HANDLE)da_handle[fd];
-  FDDEBUG(fprintf(stderr,"Closing %d (%d)\n",fd,da_handle[fd]));
+  h = da_handle[fd];
+  FDDEBUG(fprintf(stderr,"Closing %d (%ld)\n",
+		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd])));
   type=fd_type[fd];
   mt_unlock(&fd_mutex);
   switch(type)
@@ -365,19 +612,22 @@ int fd_close(FD fd)
       if(closesocket((SOCKET)h))
       {
 	errno=GetLastError();
-	FDDEBUG(fprintf(stderr,"Closing %d (%d) failed with errno=%d\n",fd,da_handle[fd],errno));
+	FDDEBUG(fprintf(stderr,"Closing %d (%ld) failed with errno=%d\n",
+			fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd]),
+			errno));
 	return -1;
       }
       break;
 
     default:
-      if(!CloseHandle((HANDLE)h))
+      if(!CloseHandle(h))
       {
 	errno=GetLastError();
 	return -1;
       }
   }
-  FDDEBUG(fprintf(stderr,"%d (%d) closed\n",fd,da_handle[fd]));
+  FDDEBUG(fprintf(stderr,"%d (%ld) closed\n",
+		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd])));
   mt_lock(&fd_mutex);
   if(fd_type[fd]<FD_NO_MORE_FREE)
   {
@@ -389,41 +639,55 @@ int fd_close(FD fd)
   return 0;
 }
 
-long fd_write(FD fd, void *buf, long len)
+PMOD_EXPORT ptrdiff_t debug_fd_write(FD fd, void *buf, ptrdiff_t len)
 {
-  DWORD ret;
-  long handle;
+  int kind;
+  HANDLE handle;
+
   mt_lock(&fd_mutex);
-  FDDEBUG(fprintf(stderr,"Writing %d bytes to %d (%d)\n",len,fd,da_handle[fd]));
-  ret=fd_type[fd];
-  handle=da_handle[fd];
+  FDDEBUG(fprintf(stderr, "Writing %d bytes to %d (%d)\n",
+		  len, fd, da_handle[fd]));
+  kind = fd_type[fd];
+  handle = da_handle[fd];
   mt_unlock(&fd_mutex);
   
-  switch(ret)
+  switch(kind)
   {
     case FD_SOCKET:
-      ret=send((SOCKET)handle, buf, len, 0);
-      if(ret<0)
       {
-	errno=WSAGetLastError();
-	FDDEBUG(fprintf(stderr,"Write on %d failed (%d)\n",fd,errno));
-	return -1;
+	ptrdiff_t ret = send((SOCKET)handle, buf,
+			     DO_NOT_WARN((int)len),
+			     0);
+	if(ret<0)
+	{
+	  errno = WSAGetLastError();
+	  FDDEBUG(fprintf(stderr, "Write on %d failed (%d)\n", fd, errno));
+	  if (errno == 1) {
+	    /* UGLY kludge */
+	    errno = WSAEWOULDBLOCK;
+	  }
+	  return -1;
+	}
+	FDDEBUG(fprintf(stderr, "Wrote %d bytes to %d)\n", len, fd));
+	return ret;
       }
-      FDDEBUG(fprintf(stderr,"Wrote %d bytes to %d)\n",len,fd));
-      return ret;
 
     case FD_CONSOLE:
     case FD_FILE:
     case FD_PIPE:
-      ret=0;
-      if(!WriteFile((HANDLE)handle, buf, len, &ret,0) && ret<=0)
       {
-	errno=GetLastError();
-	FDDEBUG(fprintf(stderr,"Write on %d failed (%d)\n",fd,errno));
-	return -1;
+	DWORD ret = 0;
+	if(!WriteFile(handle, buf,
+		      DO_NOT_WARN((DWORD)len),
+		      &ret,0) && ret<=0)
+	{
+	  errno = GetLastError();
+	  FDDEBUG(fprintf(stderr, "Write on %d failed (%d)\n", fd, errno));
+	  return -1;
+	}
+	FDDEBUG(fprintf(stderr, "Wrote %ld bytes to %d)\n", (long)ret, fd));
+	return ret;
       }
-      FDDEBUG(fprintf(stderr,"Wrote %d bytes to %d)\n",len,fd));
-      return ret;
 
     default:
       errno=ENOTSUPP;
@@ -431,14 +695,16 @@ long fd_write(FD fd, void *buf, long len)
   }
 }
 
-long fd_read(FD fd, void *to, long len)
+PMOD_EXPORT ptrdiff_t debug_fd_read(FD fd, void *to, ptrdiff_t len)
 {
   DWORD ret;
-  int rret;
-  long handle;
+  ptrdiff_t rret;
+  HANDLE handle;
 
   mt_lock(&fd_mutex);
-  FDDEBUG(fprintf(stderr,"Reading %d bytes from %d (%d) to %lx\n",len,fd,da_handle[fd],(int)(char *)to));
+  FDDEBUG(fprintf(stderr,"Reading %d bytes from %d (%d) to %lx\n",
+		  len, fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd]),
+		  PTRDIFF_T_TO_LONG((ptrdiff_t)to)));
   ret=fd_type[fd];
   handle=da_handle[fd];
   mt_unlock(&fd_mutex);
@@ -446,7 +712,9 @@ long fd_read(FD fd, void *to, long len)
   switch(ret)
   {
     case FD_SOCKET:
-      rret=recv((SOCKET)handle, to, len, 0);
+      rret=recv((SOCKET)handle, to,
+		DO_NOT_WARN((int)len),
+		0);
       if(rret<0)
       {
 	errno=WSAGetLastError();
@@ -460,7 +728,9 @@ long fd_read(FD fd, void *to, long len)
     case FD_FILE:
     case FD_PIPE:
       ret=0;
-      if(!ReadFile((HANDLE)handle, to, len, &ret,0) && ret<=0)
+      if(len && !ReadFile(handle, to,
+			  DO_NOT_WARN((DWORD)len),
+			  &ret,0) && ret<=0)
       {
 	errno=GetLastError();
 	switch(errno)
@@ -481,9 +751,11 @@ long fd_read(FD fd, void *to, long len)
   }
 }
 
-long fd_lseek(FD fd, long pos, int where)
+PMOD_EXPORT ptrdiff_t debug_fd_lseek(FD fd, ptrdiff_t pos, int where)
 {
-  long ret;
+  ptrdiff_t ret;
+  HANDLE h;
+
   mt_lock(&fd_mutex);
   if(fd_type[fd]!=FD_FILE)
   {
@@ -499,11 +771,13 @@ long fd_lseek(FD fd, long pos, int where)
     case SEEK_END: where=FILE_END; break;
   }
 #endif
-  ret=da_handle[fd];
+  h = da_handle[fd];
   mt_unlock(&fd_mutex);
 
-  ret=SetFilePointer((HANDLE)ret, pos, 0, where);
-  if(ret == 0xffffffff)
+  ret = SetFilePointer(h,
+		       DO_NOT_WARN((LONG)pos),
+		       0, where);
+  if(!~ret)
   {
     errno=GetLastError();
     return -1;
@@ -511,9 +785,12 @@ long fd_lseek(FD fd, long pos, int where)
   return ret;
 }
 
-int fd_flock(FD fd, int oper)
+PMOD_EXPORT int debug_fd_ftruncate(FD fd, ptrdiff_t len)
 {
-  long ret;
+  ptrdiff_t ret;
+  HANDLE h;
+  LONG oldfp_lo, oldfp_hi;
+
   mt_lock(&fd_mutex);
   if(fd_type[fd]!=FD_FILE)
   {
@@ -521,12 +798,49 @@ int fd_flock(FD fd, int oper)
     errno=ENOTSUPP;
     return -1;
   }
-  ret=da_handle[fd];
+  h = da_handle[fd];
+  mt_unlock(&fd_mutex);
+
+  oldfp_hi = 0;
+  oldfp_lo = SetFilePointer(h, 0, &oldfp_hi, FILE_CURRENT);
+  if(!~oldfp_lo) {
+    errno=GetLastError();
+    if(errno != NO_ERROR)
+      return -1;
+  }
+  if(!~SetFilePointer(h,
+		      DO_NOT_WARN((LONG)len),
+		      NULL, FILE_BEGIN) ||
+     !SetEndOfFile(h)) {
+    errno=GetLastError();
+    SetFilePointer(h, oldfp_lo, &oldfp_hi, FILE_BEGIN);
+    return -1;
+  }
+  if(!~SetFilePointer(h, oldfp_lo, &oldfp_hi, FILE_BEGIN)) {
+    errno=GetLastError();
+    if(errno != NO_ERROR)
+      return -1;
+  }
+  return 0;
+}
+
+PMOD_EXPORT int debug_fd_flock(FD fd, int oper)
+{
+  long ret;
+  HANDLE h;
+  mt_lock(&fd_mutex);
+  if(fd_type[fd]!=FD_FILE)
+  {
+    mt_unlock(&fd_mutex);
+    errno=ENOTSUPP;
+    return -1;
+  }
+  h = da_handle[fd];
   mt_unlock(&fd_mutex);
 
   if(oper & fd_LOCK_UN)
   {
-    ret=UnlockFile((HANDLE)ret,
+    ret=UnlockFile(h,
 		   0,
 		   0,
 		   0xffffffff,
@@ -544,7 +858,7 @@ int fd_flock(FD fd, int oper)
     if(oper & fd_LOCK_UN)
       flags|=LOCKFILE_FAIL_IMMEDIATELY;
 
-    ret=LockFileEx((HANDLE)ret,
+    ret=LockFileEx(h,
 		   flags,
 		   0,
 		   0xffffffff,
@@ -567,15 +881,17 @@ static long convert_filetime_to_time_t(FILETIME tmp)
   t=tmp.dwHighDateTime * pow(2.0,32.0) + (double)tmp.dwLowDateTime;
   t/=10000000.0;
   t-=11644473600.0;
-  return (long)floor(t);
+  return DO_NOT_WARN((long)floor(t));
 }
 
-int fd_fstat(FD fd, struct stat *s)
+PMOD_EXPORT int debug_fd_fstat(FD fd, struct stat *s)
 {
   DWORD x;
 
   FILETIME c,a,m;
-  FDDEBUG(fprintf(stderr,"fstat on %d (%d)\n",fd,da_handle[fd]));
+  mt_lock(&fd_mutex);
+  FDDEBUG(fprintf(stderr, "fstat on %d (%ld)\n",
+		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd])));
   if(fd_type[fd]!=FD_FILE)
   {
     errno=ENOTSUPP;
@@ -593,20 +909,21 @@ int fd_fstat(FD fd, struct stat *s)
       break;
 
     default:
-      switch(GetFileType((HANDLE)da_handle[fd]))
+      switch(GetFileType(da_handle[fd]))
       {
 	default:
 	case FILE_TYPE_UNKNOWN: s->st_mode=0;        break;
 	case FILE_TYPE_DISK:
 	  s->st_mode=S_IFREG; 
-	  s->st_size=GetFileSize((HANDLE)da_handle[fd],&x);
+	  s->st_size=GetFileSize(da_handle[fd],&x);
 	  if(x)
 	  {
 	    s->st_size=0x7fffffff;
 	  }
-	  if(!GetFileTime((HANDLE)da_handle[fd], &c, &a, &m))
+	  if(!GetFileTime(da_handle[fd], &c, &a, &m))
 	  {
 	    errno=GetLastError();
+            mt_unlock(&fd_mutex);
 	    return -1;
 	  }
 	  s->st_ctime=convert_filetime_to_time_t(c);
@@ -614,30 +931,81 @@ int fd_fstat(FD fd, struct stat *s)
 	  s->st_mtime=convert_filetime_to_time_t(m);
 	  break;
 	case FILE_TYPE_CHAR:    s->st_mode=S_IFCHR;  break;
-	case FILE_TYPE_PIPE:    s->st_mode=S_IFFIFO; break;
+	case FILE_TYPE_PIPE:    s->st_mode=S_IFIFO; break;
       }
   }
   s->st_mode |= 0666;
+  mt_unlock(&fd_mutex);
   return 0;
 }
 
-int fd_select(int fds, FD_SET *a, FD_SET *b, FD_SET *c, struct timeval *t)
+
+#ifdef FD_DEBUG
+static void dump_FDSET(FD_SET *x, int fds)
+{
+  if(x)
+  {
+    int e, first=1;
+    fprintf(stderr,"[");
+    for(e=0;e<fds;e++)
+    {
+      if(FD_ISSET(da_handle[e],x))
+      {
+	if(!first) fprintf(stderr,",",e);
+	fprintf(stderr,"%d",e);
+	first=0;
+      }
+    }
+    fprintf(stderr,"]");
+  }else{
+    fprintf(stderr,"0");
+  }
+}
+#endif
+
+/* FIXME:
+ * select with no fds should call Sleep()
+ * If the backend works correctly, fds is zero when there are no fds.
+ * /Hubbe
+ */
+PMOD_EXPORT int debug_fd_select(int fds, FD_SET *a, FD_SET *b, FD_SET *c, struct timeval *t)
 {
   int ret;
+
+  FDDEBUG(
+    int e;
+    fprintf(stderr,"Select(%d,",fds);
+    dump_FDSET(a,fds);
+    dump_FDSET(b,fds);
+    dump_FDSET(c,fds);
+    fprintf(stderr,",(%ld,%06ld));\n", (long) t->tv_sec,(long) t->tv_usec);
+    )
+
   ret=select(fds,a,b,c,t);
   if(ret==SOCKET_ERROR)
   {
     errno=WSAGetLastError();
+    FDDEBUG(fprintf(stderr,"select->%d, errno=%d\n",ret,errno));
     return -1;
   }
+
+  FDDEBUG(
+    fprintf(stderr,"    ->(%d,",fds);
+    dump_FDSET(a,fds);
+    dump_FDSET(b,fds);
+    dump_FDSET(c,fds);
+    fprintf(stderr,",(%ld,%06ld));\n", (long) t->tv_sec,(long) t->tv_usec);
+    )
+
   return ret;
 }
 
 
-int fd_ioctl(FD fd, int cmd, void *data)
+PMOD_EXPORT int debug_fd_ioctl(FD fd, int cmd, void *data)
 {
   int ret;
-  FDDEBUG(fprintf(stderr,"ioctl(%d (%d,%d,%p)\n",fd,da_handle[fd],cmd,data));
+  FDDEBUG(fprintf(stderr,"ioctl(%d (%ld,%d,%p)\n",
+		  fd, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[fd]), cmd, data));
   switch(fd_type[fd])
   {
     case FD_SOCKET:
@@ -657,7 +1025,7 @@ int fd_ioctl(FD fd, int cmd, void *data)
 }
 
 
-FD fd_dup(FD from)
+PMOD_EXPORT FD debug_fd_dup(FD from)
 {
   FD fd;
   HANDLE x,p=GetCurrentProcess();
@@ -665,7 +1033,7 @@ FD fd_dup(FD from)
   if(fd_type[from]>=FD_NO_MORE_FREE)
     fatal("fd_dup() on file which is not open!\n");
 #endif
-  if(!DuplicateHandle(p,(HANDLE)da_handle[from],p,&x,NULL,0,DUPLICATE_SAME_ACCESS))
+  if(!DuplicateHandle(p,da_handle[from],p,&x,0,0,DUPLICATE_SAME_ACCESS))
   {
     errno=GetLastError();
     return -1;
@@ -675,17 +1043,18 @@ FD fd_dup(FD from)
   fd=first_free_handle;
   first_free_handle=fd_type[fd];
   fd_type[fd]=fd_type[from];
-  da_handle[fd]=(long)x;
+  da_handle[fd] = x;
   mt_unlock(&fd_mutex);
   
-  FDDEBUG(fprintf(stderr,"Dup %d (%d) to %d (%d)\n",from,da_handle[from],fd,x));
+  FDDEBUG(fprintf(stderr,"Dup %d (%ld) to %d (%d)\n",
+		  from, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[from]), fd, x));
   return fd;
 }
 
-FD fd_dup2(FD from, FD to)
+PMOD_EXPORT FD debug_fd_dup2(FD from, FD to)
 {
   HANDLE x,p=GetCurrentProcess();
-  if(!DuplicateHandle(p,(HANDLE)da_handle[from],p,&x,NULL,0,DUPLICATE_SAME_ACCESS))
+  if(!DuplicateHandle(p,da_handle[from],p,&x,0,0,DUPLICATE_SAME_ACCESS))
   {
     errno=GetLastError();
     return -1;
@@ -694,7 +1063,7 @@ FD fd_dup2(FD from, FD to)
   mt_lock(&fd_mutex);
   if(fd_type[to] < FD_NO_MORE_FREE)
   {
-    if(!CloseHandle((HANDLE)da_handle[to]))
+    if(!CloseHandle(da_handle[to]))
     {
       errno=GetLastError();
       mt_unlock(&fd_mutex);
@@ -712,16 +1081,73 @@ FD fd_dup2(FD from, FD to)
     }
   }
   fd_type[to]=fd_type[from];
-  da_handle[to]=(long)x;
+  da_handle[to] = x;
   mt_unlock(&fd_mutex);
 
-  FDDEBUG(fprintf(stderr,"Dup2 %d (%d) to %d (%d)\n",from,da_handle[from],to,x));
+  FDDEBUG(fprintf(stderr,"Dup2 %d (%d) to %d (%d)\n",
+		  from, PTRDIFF_T_TO_LONG((ptrdiff_t)da_handle[from]), to, x));
 
   return to;
 }
 
 #endif /* HAVE_WINSOCK_H */
 
+#ifdef EMULATE_DIRECT
+PMOD_EXPORT DIR *opendir(char *dir)
+{
+  ptrdiff_t len=strlen(dir);
+  char *foo;
+  DIR *ret=(DIR *)malloc(sizeof(DIR) + len+5);
+  if(!ret)
+  {
+    errno=ENOMEM;
+    return 0;
+  }
+  foo=sizeof(DIR) + (char *)ret;
+  MEMCPY(foo, dir, len);
+
+  if(len && foo[len-1]!='/') foo[len++]='/';
+  foo[len++]='*';
+  foo[len]=0;
+/*  fprintf(stderr,"opendir(%s)\n",foo); */
+
+  /* This may require appending a slash and a star... */
+  ret->h=FindFirstFile( (LPCTSTR) foo, & ret->find_data);
+  if(ret->h == DO_NOT_WARN(INVALID_HANDLE_VALUE))
+  {
+    errno=ENOENT;
+    free((char *)ret);
+    return 0;
+  }
+  ret->first=1;
+  return ret;
+}
+
+PMOD_EXPORT int readdir_r(DIR *dir, struct direct *tmp ,struct direct **d)
+{
+  if(dir->first)
+  {
+    *d=&dir->find_data;
+    dir->first=0;
+    return 0;
+  }else{
+    if(FindNextFile(dir->h,tmp))
+    {
+      *d=tmp;
+      return 0;
+    }
+    *d=0;
+    return 0;
+  }
+}
+
+PMOD_EXPORT void closedir(DIR *dir)
+{
+  FindClose(dir->h);
+  free((char *)dir);
+}
+#endif
+
 #if 0
 
 #ifdef FD_LINEAR
