@@ -1,7 +1,7 @@
 #include "global.h"
-RCSID("$Id: png.c,v 1.21 1998/08/26 16:14:24 per Exp $");
+RCSID("$Id: png.c,v 1.41 2000/12/05 21:08:27 per Exp $");
 
-#include "config.h"
+#include "image_machine.h"
 
 #include "pike_macros.h"
 #include "object.h"
@@ -11,13 +11,18 @@ RCSID("$Id: png.c,v 1.21 1998/08/26 16:14:24 per Exp $");
 #include "threads.h"
 #include "array.h"
 #include "mapping.h"
-#include "error.h"
+#include "pike_error.h"
 #include "stralloc.h"
 #include "dynamic_buffer.h"
+#include "builtin_functions.h"
+#include "operators.h"
 
 #include "image.h"
 #include "colortable.h"
 
+/* MUST BE INCLUDED LAST */
+#include "module_magic.h"
+
 extern struct program *image_colortable_program;
 extern struct program *image_program;
 
@@ -34,11 +39,6 @@ static struct pike_string *param_bpp;
 static struct pike_string *param_background;
 
 
-void f_add(INT32 args);
-void f_aggregate(INT32 args);
-void f_index(INT32 args);
-
-
 /*
 **! module Image
 **! submodule PNG
@@ -47,13 +47,13 @@ void f_index(INT32 args);
 **!	This module uses <tt>zlib</tt>.
 */
 
-static INLINE void push_nbo_32bit(unsigned long x)
+static INLINE void push_nbo_32bit(size_t x)
 {
    char buf[4];
-   buf[0]=(char)(x>>24);
-   buf[1]=(char)(x>>16);
-   buf[2]=(char)(x>>8);
-   buf[3]=(char)(x);
+   buf[0] = DO_NOT_WARN((char)(x>>24));
+   buf[1] = DO_NOT_WARN((char)(x>>16));
+   buf[2] = DO_NOT_WARN((char)(x>>8));
+   buf[3] = DO_NOT_WARN((char)(x));
    push_string(make_shared_binary_string(buf,4));
 }
 
@@ -86,7 +86,7 @@ static INLINE INT32 call_gz_crc32(INT32 args)
    INT32 z;
    apply_svalue(&gz_crc32,args);
    if (sp[-1].type!=T_INT)
-      error("Image.PNG: internal error (not integer from Gz.crc32)\n");
+      Pike_error("Image.PNG: internal error (not integer from Gz.crc32)\n");
    z=sp[-1].u.integer;
    pop_stack();
    return z;
@@ -130,7 +130,7 @@ static void png_decompress(int style)
    struct object *o;
 
    if (style)
-      error("internal error: illegal decompression style %d\n",style);
+      Pike_error("Internal error: illegal decompression style %d\n",style);
    
    o=clone_object(gz_inflate,0);
    apply(o,"inflate",1);
@@ -142,7 +142,7 @@ static void png_compress(int style)
    struct object *o;
 
    if (style)
-      error("internal error: illegal decompression style %d\n",style);
+      Pike_error("Internal error: illegal decompression style %d\n",style);
    
    push_int(8);
    o=clone_object(gz_deflate,1);
@@ -170,11 +170,11 @@ static void image_png__chunk(INT32 args)
    if (args!=2 ||
        sp[-args].type!=T_STRING ||
        sp[1-args].type!=T_STRING)
-      error("Image.PNG.chunk: Illegal argument(s)\n");
+      Pike_error("Image.PNG.chunk: Illegal argument(s)\n");
    
    a=sp[-args].u.string;
    if (a->len!=4)
-      error("Image.PNG.chunk: Type string not 4 characters\n");
+      Pike_error("Image.PNG.chunk: Type string not 4 characters\n");
    b=sp[1-args].u.string;
    pop_n_elems(args-2);
    sp-=2;
@@ -210,14 +210,15 @@ static void image_png___decode(INT32 args)
 {
    int nocrc=0;
    unsigned char *data;
-   unsigned long len;
+   size_t len;
    struct pike_string *str;
    int n=0;
+   ONERROR uwp;
 
    if (args<1) 
-      error("Image.PNG.__decode: too few arguments\n");
+      Pike_error("Image.PNG.__decode: too few arguments\n");
    if (sp[-args].type!=T_STRING)
-      error("Image.PNG.__decode: illegal argument 1\n");
+      Pike_error("Image.PNG.__decode: illegal argument 1\n");
    
    if (args==2 &&
        (sp[1-args].type!=T_INT ||
@@ -227,6 +228,7 @@ static void image_png___decode(INT32 args)
    add_ref(str=sp[-args].u.string);
    data=(unsigned char*)str->str;
    len=str->len;
+   SET_ONERROR(uwp,do_free_string,str);
 
    pop_n_elems(args);
 
@@ -240,6 +242,7 @@ static void image_png___decode(INT32 args)
        data[6]!=26 ||
        data[7]!=10)
    {
+      UNSET_ONERROR(uwp);
       free_string(str);
       push_int(0);
       return;
@@ -275,6 +278,7 @@ static void image_png___decode(INT32 args)
       data+=x+4;
    }
 
+   UNSET_ONERROR(uwp);
    free_string(str);
    f_aggregate(n);
 }
@@ -374,7 +378,7 @@ static void image_png___decode(INT32 args)
 */
 
 static struct pike_string *_png_unfilter(unsigned char *data,
-					 INT32 len,
+					 size_t len,
 					 int xsize,int ysize,
 					 int filter,int type,
 					 int bpp,
@@ -415,7 +419,7 @@ static struct pike_string *_png_unfilter(unsigned char *data,
 #if 0
       fprintf(stderr,
 	      "%05d 0x%08lx %02x %02x %02x > %02x < %02x %02x %02x "
-	      "len=%d xsize=%d sbb=%d next=0x%08lx d=0x%lx nd=0x%lx\n",
+	      "len=%ld xsize=%d sbb=%d next=0x%08lx d=0x%lx nd=0x%lx\n",
 	      ysize+1,(unsigned long)s,
 	      s[-3],s[-2],s[-1],s[0],s[1],s[2],s[3],
 	      len,xsize,sbb,
@@ -501,7 +505,7 @@ static struct pike_string *_png_unfilter(unsigned char *data,
 	    
 	    break;
 	 default:
-	    error("Image.PNG._decode: unsupported filter %d\n",s[-1]);
+	    Pike_error("Image.PNG._decode: unsupported filter %d\n",s[-1]);
       }
    }
 }
@@ -510,9 +514,9 @@ static int _png_write_rgb(rgb_group *w1,
 			  rgb_group *wa1,
 			  int type,int bpp,
 			  unsigned char *s,
-			  unsigned long len,
+			  size_t len,
 			  unsigned long width,
-			  unsigned long n,
+			  size_t n,
 			  struct neo_colortable *ct,
 			  struct pike_string *trns)
 {
@@ -526,10 +530,10 @@ static int _png_write_rgb(rgb_group *w1,
    rgb_group *d1=w1;
    rgb_group *da1=wa1;
 
-   unsigned long n0=n;
+   size_t n0=n;
 
    unsigned long x;
-   int mz;
+   ptrdiff_t mz;
 
    /* write stuff to d1 */
 
@@ -612,7 +616,7 @@ static int _png_write_rgb(rgb_group *w1,
 	       break;
 	    default:
 	       free(wa1); free(w1);
-	       error("Image.PNG->_decode: Unsupported color type/bit depth %d (grey)/%d bit.\n",
+	       Pike_error("Image.PNG->_decode: Unsupported color type/bit depth %d (grey)/%d bit.\n",
 		     type,bpp);
 	 }
 	 if (trns && trns->len==2)
@@ -653,7 +657,7 @@ static int _png_write_rgb(rgb_group *w1,
 	       break;
 	    default:
 	       free(wa1); free(w1);
-	       error("Image.PNG->_decode: Unsupported color type/bit depth %d (rgb)/%d bit.\n",
+	       Pike_error("Image.PNG->_decode: Unsupported color type/bit depth %d (rgb)/%d bit.\n",
 		     type,bpp);
 	 }
 	 if (trns && trns->len==6)
@@ -674,20 +678,20 @@ static int _png_write_rgb(rgb_group *w1,
 	 {
 	    free(w1);
 	    free(wa1);
-	    error("Image.PNG->decode: No palette (PLTE entry), but color type (3) needs one\n");
+	    Pike_error("Image.PNG->decode: No palette (PLTE entry), but color type (3) needs one\n");
 	 }
 	 if (ct->type!=NCT_FLAT)
 	 {
 	    free(w1);
 	    free(wa1);
-	    error("Image.PNG->decode: Internal error (created palette isn't flat)\n");
+	    Pike_error("Image.PNG->decode: Internal error (created palette isn't flat)\n");
 	 }
 	 mz=ct->u.flat.numentries;
 	 if (mz==0)
 	 {
 	    free(w1);
 	    free(wa1);
-	    error("Image.PNG->decode: palette is zero entries long; need at least one color.\n");
+	    Pike_error("Image.PNG->decode: palette is zero entries long; need at least one color.\n");
 	 }
 
 #define CUTPLTE(X,Z) (((X)>=(Z))?0:(X))
@@ -699,14 +703,14 @@ static int _png_write_rgb(rgb_group *w1,
 	       if (!trns)
 		  while (n)
 		  {
-		     if (x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>7)&1,mz)].color;
-		     if (x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>6)&1,mz)].color;
-		     if (x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>5)&1,mz)].color;
-		     if (x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>4)&1,mz)].color;
-		     if (x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>3)&1,mz)].color;
-		     if (x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>2)&1,mz)].color;
-		     if (x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>1)&1,mz)].color;
-		     if (x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE((*s)&1,mz)].color;
+		    if (n&&x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>7)&1,mz)].color;
+		     if (n&&x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>6)&1,mz)].color;
+		     if (n&&x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>5)&1,mz)].color;
+		     if (n&&x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>4)&1,mz)].color;
+		     if (n&&x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>3)&1,mz)].color;
+		     if (n&&x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>2)&1,mz)].color;
+		     if (n&&x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>1)&1,mz)].color;
+		     if (n&&x) n--,x--,*(d1++)=ct->u.flat.entries[CUTPLTE((*s)&1,mz)].color;
 		     s++;
 		     if (!x) x=width;
 		  }
@@ -745,10 +749,10 @@ static int _png_write_rgb(rgb_group *w1,
 	       if (!trns)
 		  while (n)
 		  {
-		     if (x) x--,n--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>6)&3,mz)].color;
-		     if (x) x--,n--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>4)&3,mz)].color;
-		     if (x) x--,n--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>2)&3,mz)].color;
-		     if (x) x--,n--,*(d1++)=ct->u.flat.entries[CUTPLTE((*s)&3,mz)].color;
+		     if (n&&x) x--,n--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>6)&3,mz)].color;
+		     if (n&&x) x--,n--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>4)&3,mz)].color;
+		     if (n&&x) x--,n--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>2)&3,mz)].color;
+		     if (n&&x) x--,n--,*(d1++)=ct->u.flat.entries[CUTPLTE((*s)&3,mz)].color;
 		     s++;
 		     if (!x) x=width;
 		  }
@@ -787,8 +791,8 @@ static int _png_write_rgb(rgb_group *w1,
 	       if (!trns)
 		  while (n)
 		  {
-		     if (x) x--,n--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>4)&15,mz)].color;
-		     if (x) x--,n--,*(d1++)=ct->u.flat.entries[CUTPLTE((*s)&15,mz)].color;
+		     if (n&&x) x--,n--,*(d1++)=ct->u.flat.entries[CUTPLTE(((*s)>>4)&15,mz)].color;
+		     if (n&&x) x--,n--,*(d1++)=ct->u.flat.entries[CUTPLTE((*s)&15,mz)].color;
 		     s++;
 		     if (!x) x=width;
 		  }
@@ -854,10 +858,10 @@ static int _png_write_rgb(rgb_group *w1,
 	       
 	    default:
 	       free(w1); free(wa1);
-	       error("Image.PNG->_decode: Unsupported color type/bit depth %d (palette)/%d bit.\n",
+	       Pike_error("Image.PNG->_decode: Unsupported color type/bit depth %d (palette)/%d bit.\n",
 		     type,bpp);
 	 }
-	 return !trns; /* alpha channel if trns chunk */
+	 return !!trns; /* alpha channel if trns chunk */
 
       case 4: /* 8 or 16 bit grey,a */
 	 switch (bpp)
@@ -888,7 +892,7 @@ static int _png_write_rgb(rgb_group *w1,
 	       break;
 	    default:
 	       free(wa1); free(w1);
-	       error("Image.PNG->_decode: Unsupported color type/bit depth %d (grey+a)/%d bit.\n",
+	       Pike_error("Image.PNG->_decode: Unsupported color type/bit depth %d (grey+a)/%d bit.\n",
 		     type,bpp);
 	 }
 	 return 1; /* alpha channel */
@@ -897,7 +901,7 @@ static int _png_write_rgb(rgb_group *w1,
 	 switch (bpp)
 	 {
 	    case 8:
-	       if (n>len/3) n=len/3;
+	       if (n>len/4) n=len/4;
 	       while (n)
 	       {
 		  d1->r=*(s++);
@@ -910,7 +914,7 @@ static int _png_write_rgb(rgb_group *w1,
 	       }
 	       break;
 	    case 16:
-	       if (n>len/6) n=len/6;
+	       if (n>len/8) n=len/8;
 	       while (n)
 	       {
 		  d1->r=*(s++);
@@ -926,16 +930,16 @@ static int _png_write_rgb(rgb_group *w1,
 	       break;
 	    default:
 	       free(wa1); free(w1);
-	       error("Image.PNG->_decode: Unsupported color type/bit depth %d(rgba)/%d bit.\n",
+	       Pike_error("Image.PNG->_decode: Unsupported color type/bit depth %d(rgba)/%d bit.\n",
 		     type,bpp);
 	 }
 	 return 1; /* alpha channel */
       default:
 	 free(wa1); free(w1);
-	 error("Image.PNG->_decode: Unknown color type %d (bit depth %d).\n",
+	 Pike_error("Image.PNG->_decode: Unknown color type %d (bit depth %d).\n",
 	       type,bpp);
    }
-   error("Image.PNG->_decode: illegal state\n");
+   Pike_error("Image.PNG->_decode: illegal state\n");
    return 0; /* stupid */
 }
 
@@ -953,7 +957,7 @@ static struct png_interlace adam7[8]=
   {0,2,1,2},
   {1,2,0,1} };
 
-static void image_png__decode(INT32 args)
+static void img_png_decode(INT32 args,int header_only)
 {
    struct array *a;
    struct mapping *m;
@@ -975,7 +979,7 @@ static void image_png__decode(INT32 args)
    } ihdr={-1,-1,-1,0,-1,-1,-1};
 
    if (args<1) 
-      error("Image.PNG._decode: too few arguments\n");
+      Pike_error("Image.PNG._decode: too few arguments\n");
 
    m=allocate_mapping(10);
    push_mapping(m);
@@ -983,7 +987,7 @@ static void image_png__decode(INT32 args)
    if (args>=2)
    {
       if (sp[1-args-1].type!=T_MAPPING)
-	 error("Image.PNG._decode: illegal argument 2\n");
+	 Pike_error("Image.PNG._decode: illegal argument 2\n");
 
       push_svalue(sp+1-args-1);
       ref_push_string(param_palette);
@@ -993,7 +997,7 @@ static void image_png__decode(INT32 args)
 	 case T_OBJECT:
 	    push_string(make_shared_string("cast"));
 	    if (sp[-1].type==T_INT)
-	       error("Image.PNG._decode: illegal value of option \"palette\"\n");
+	       Pike_error("Image.PNG._decode: illegal value of option \"palette\"\n");
 	    f_index(2);
 	    push_string(make_shared_string("array"));
 	    f_call_function(2);
@@ -1004,7 +1008,7 @@ static void image_png__decode(INT32 args)
 	    ct=(struct neo_colortable*)get_storage(sp[-1].u.object,
 						   image_colortable_program);
 	    if (!ct)
-	       error("Image.PNG._decode: internal error: cloned colortable isn't colortable\n");
+	       Pike_error("Image.PNG._decode: internal error: cloned colortable isn't colortable\n");
 	    ref_push_string(param_palette);
 	    mapping_insert(m,sp-1,sp-2);
 	    pop_n_elems(2);
@@ -1012,7 +1016,7 @@ static void image_png__decode(INT32 args)
 	 case T_INT:
 	    pop_n_elems(1);
 	 default:
-	    error("Image.PNG._decode: illegal value of option \"palette\"\n");
+	    Pike_error("Image.PNG._decode: illegal value of option \"palette\"\n");
       }
    }
 
@@ -1026,33 +1030,33 @@ static void image_png__decode(INT32 args)
       push_int(1); /* no care crc */
       image_png___decode(2);
       if (sp[-1].type!=T_ARRAY)
-	 error("Image.PNG._decode: Not PNG data\n");
+	 Pike_error("Image.PNG._decode: Not PNG data\n");
    }
    else if (sp[-1].type!=T_ARRAY)
-      error("Image.PNG._decode: Illegal argument 1\n");
+      Pike_error("Image.PNG._decode: Illegal argument 1\n");
 
    a=sp[-1].u.array;
 
    for (i=0; i<a->size; i++)
    {
-      struct array *b;
+      struct array *b = NULL;
       unsigned char *data;
-      unsigned long len;
+      size_t len;
 
       if (a->item[i].type!=T_ARRAY ||
 	  (b=a->item[i].u.array)->size!=3 ||
 	  b->item[0].type!=T_STRING ||
 	  b->item[1].type!=T_STRING ||
 	  b->item[0].u.string->len!=4)
-	 error("Image.PNG._decode: Illegal stuff in array index %d\n",i);
+	 Pike_error("Image.PNG._decode: Illegal stuff in array index %d\n",i);
 
-      data=(unsigned char*)b->item[1].u.string->str;
-      len=(unsigned long)b->item[1].u.string->len;
+      data = (unsigned char *)b->item[1].u.string->str;
+      len = (size_t)b->item[1].u.string->len;
 
       if (!i &&
 	  int_from_32bit((unsigned char*)b->item[0].u.string->str)
 	  != 0x49484452 )
-	 error("Imge.PNG.decode: first chunk isn't IHDR\n");
+	 Pike_error("Imge.PNG.decode: first chunk isn't IHDR\n");
 
       switch (int_from_32bit((unsigned char*)b->item[0].u.string->str))
       {
@@ -1060,7 +1064,7 @@ static void image_png__decode(INT32 args)
          case 0x49484452: /* IHDR */
 	    /* header info */
 	    if (b->item[1].u.string->len!=13)
-	       error("Image.PNG._decode: illegal header (IHDR chunk)\n");
+	       Pike_error("Image.PNG._decode: illegal header (IHDR chunk)\n");
 
 	    ihdr.width=int_from_32bit(data+0);
 	    ihdr.height=int_from_32bit(data+4);
@@ -1096,8 +1100,10 @@ static void image_png__decode(INT32 args)
 
          case 0x49444154: /* IDAT */
 	    /* compressed image data. push, n++ */
+	    if (header_only) break;
+
 	    if (ihdr.compression!=0)
-	       error("Image.PNG._decode: unknown compression (%d)\n",
+	       Pike_error("Image.PNG._decode: unknown compression (%d)\n",
 		     ihdr.compression);
 
 	    ref_push_string(b->item[1].u.string);
@@ -1147,17 +1153,18 @@ static void image_png__decode(INT32 args)
 		     push_int(ct->u.flat.entries[(int)b->item[1].u.string->str[0]].color.g);
 		     push_int(ct->u.flat.entries[(int)b->item[1].u.string->str[0]].color.b);
 		  }
-		  break;
+                  break;
 	       default:
-		  if (b->item[1].u.string->len!=6)
+		  if (b->item[1].u.string->len==6)
 		  {
 		     int j;
 		     for (j=0; j<3; j++)
 		     {
-			int z=_png_c16(b->item[1].u.string->str[j],ihdr.bpp);
+			int z=_png_c16(b->item[1].u.string->str[j*2],ihdr.bpp);
 			push_int(z);
 		     }
-		  }
+		  } else
+                    continue;
 		  break;
 	    }
 	    f_aggregate(3);
@@ -1190,6 +1197,8 @@ static void image_png__decode(INT32 args)
       }
    }
 
+   if (header_only)  goto header_stuff;
+
    /* on stack: mapping   nstring */
 
    /* IDAT stuff on stack, now */
@@ -1200,21 +1209,21 @@ static void image_png__decode(INT32 args)
 
    if (ihdr.type==-1)
    {
-      error("Image.PNG._decode: missing header (IHDR chunk)\n");
+      Pike_error("Image.PNG._decode: missing header (IHDR chunk)\n");
    }
    if (ihdr.type==3 && !ct)
    {
-      error("Image.PNG._decode: missing palette (PLTE chunk)\n");
+      Pike_error("Image.PNG._decode: missing palette (PLTE chunk)\n");
    }
    
    if (ihdr.compression==0)
    {
       png_decompress(ihdr.compression);
       if (sp[-1].type!=T_STRING)
-	 error("Image.PNG._decode: got wierd stuff from decompression\n");
+	 Pike_error("Image.PNG._decode: got wierd stuff from decompression\n");
    }
    else
-      error("Image.PNG._decode: illegal compression type 0x%02x\n",
+      Pike_error("Image.PNG._decode: illegal compression type 0x%02x\n",
 	    ihdr.compression);
 
    fs=sp[-1].u.string; 
@@ -1227,13 +1236,13 @@ static void image_png__decode(INT32 args)
 
    w1=d1=malloc(sizeof(rgb_group)*ihdr.width*ihdr.height);
    if (!d1)
-      error("Image.PNG._decode: Out of memory\n");
+      Pike_error("Image.PNG._decode: Out of memory\n");
 
    wa1=da1=malloc(sizeof(rgb_group)*ihdr.width*ihdr.height);
    if (!da1)
    {
       free(d1);
-      error("Image.PNG._decode: Out of memory\n");
+      Pike_error("Image.PNG._decode: Out of memory\n");
    }
 
    /* --- interlace decoding --- */
@@ -1270,7 +1279,7 @@ static void image_png__decode(INT32 args)
 	    if (wa1) free(wa1); 
 	    if (ta1) free(ta1); 
 	    if (ta1) free(t1); 
-	    error("Image.PNG->_decode: out of memory (close one)\n");
+	    Pike_error("Image.PNG->_decode: out of memory (close one)\n");
 	 }
 	 /* loop over adam7 interlace's 
 	    and write them to the arena */
@@ -1325,7 +1334,7 @@ static void image_png__decode(INT32 args)
 	 break;
       default:
 	 free(w1); if (wa1) free(wa1);
-	 error("Image.PNG._decode: Unknown interlace type\n");
+	 Pike_error("Image.PNG._decode: Unknown interlace type\n");
    }
 
 
@@ -1356,6 +1365,8 @@ static void image_png__decode(INT32 args)
       pop_n_elems(2);
    }
 
+header_stuff:
+
    ref_push_string(param_type);
    push_int(ihdr.type);
    mapping_insert(m,sp-2,sp-1);
@@ -1413,7 +1424,7 @@ static void image_png__decode(INT32 args)
 
 static void image_png_encode(INT32 args)
 {
-   struct image *img,*alpha=NULL;
+   struct image *img = NULL, *alpha = NULL;
    rgb_group *s,*sa=NULL;
    struct neo_colortable *ct=NULL;
 
@@ -1421,38 +1432,37 @@ static void image_png_encode(INT32 args)
    char buf[20];
    
    if (!args)
-      error("Image.PNG.encode: too few arguments\n");
+      Pike_error("Image.PNG.encode: too few arguments\n");
    
    if (sp[-args].type!=T_OBJECT ||
        !(img=(struct image*)
 	 get_storage(sp[-args].u.object,image_program)))
-      error("Image.PNG.encode: illegal argument 1\n");
+      Pike_error("Image.PNG.encode: illegal argument 1\n");
    
    if (!img->img)
-      error("Image.PNG.encode: no image\n");
+      Pike_error("Image.PNG.encode: no image\n");
 
    if (args>1)
    {
       if (sp[1-args].type!=T_MAPPING)
-	 error("Image.PNG.encode: illegal argument 2\n");
+	 Pike_error("Image.PNG.encode: illegal argument 2\n");
       
       push_svalue(sp+1-args);
       ref_push_string(param_alpha);
       f_index(2);
-      if (!(sp[-1].type==T_INT 
-	    && sp[-1].subtype==NUMBER_UNDEFINED))
+      if ( !IS_ZERO( sp - 1 ) )
 	 if (sp[-1].type!=T_OBJECT ||
 	     !(alpha=(struct image*)
 	       get_storage(sp[-1].u.object,image_program)))
-	    error("Image.PNG.encode: option (arg 2) \"alpha\" has illegal type\n");
+	    Pike_error("Image.PNG.encode: option (arg 2) \"alpha\" has illegal type\n");
       pop_stack();
 
       if (alpha &&
 	  (alpha->xsize!=img->xsize ||
 	   alpha->ysize!=img->ysize))
-	 error("Image.PNG.encode option (arg 2) \"alpha\"; images differ in size\n");
+	 Pike_error("Image.PNG.encode option (arg 2) \"alpha\"; images differ in size\n");
       if (alpha && !alpha->img)
-	 error("Image.PNG.encode option (arg 2) \"alpha\"; no image\n");
+	 Pike_error("Image.PNG.encode option (arg 2) \"alpha\"; no image\n");
 
       push_svalue(sp+1-args);
       ref_push_string(param_palette); 
@@ -1462,7 +1472,7 @@ static void image_png_encode(INT32 args)
 	 if (sp[-1].type!=T_OBJECT ||
 	     !(ct=(struct neo_colortable*)
 	       get_storage(sp[-1].u.object,image_colortable_program)))
-	    error("Image.PNG.encode: option (arg 2) \"palette\" has illegal type\n");
+	    Pike_error("Image.PNG.encode: option (arg 2) \"palette\" has illegal type\n");
       pop_stack();
 
    }
@@ -1474,10 +1484,10 @@ static void image_png_encode(INT32 args)
 
    if (ct)
    {
-      int sz;
-      sz=image_colortable_size(ct);
+      ptrdiff_t sz;
+      sz = image_colortable_size(ct);
       if (sz>256)
-	 error("Image.PNG.encode: palette size to large; "
+	 Pike_error("Image.PNG.encode: palette size to large; "
 	       "PNG doesn't support bigger palettes then 256 colors\n");
       if (sz>16) bpp=8;
       else if (sz>4) bpp=4;
@@ -1518,13 +1528,13 @@ static void image_png_encode(INT32 args)
    if (alpha) sa=alpha->img;
    if (ct)
       if (alpha)
-	 error("Image.PNG.encode: colortable and alpha channel not supported at the same time\n");
+	 Pike_error("Image.PNG.encode: colortable and alpha channel not supported at the same time\n");
       else
       {
 	 unsigned char *tmp=malloc(img->xsize*img->ysize),*ts;
 
 	 if (!tmp)
-	    error("Image.PNG.encode: out of memory\n");
+	    Pike_error("Image.PNG.encode: out of memory\n");
 	 image_colortable_index_8bit_image(ct,img->img,tmp,
 					   img->xsize*img->ysize,img->xsize);
 	 ts=tmp;
@@ -1602,6 +1612,16 @@ static void image_png_encode(INT32 args)
    sp++;
 }
 
+static void image_png__decode(INT32 args)
+{
+   img_png_decode(args,0);
+}
+
+static void image_png_decode_header(INT32 args)
+{
+   img_png_decode(args,1);
+}
+
 /*
 **! method object decode(string data)
 **! method object decode(string data, mapping options)
@@ -1620,7 +1640,7 @@ static void image_png_encode(INT32 args)
 static void image_png_decode(INT32 args)
 {
    if (!args)
-      error("Image.PNG.decode: missing argument(s)\n");
+      Pike_error("Image.PNG.decode: missing argument(s)\n");
    
    image_png__decode(args);
    push_string(make_shared_string("image"));
@@ -1631,7 +1651,7 @@ static void image_png_decode_alpha(INT32 args)
 {
    struct svalue s;
    if (!args)
-      error("Image.PNG.decode: missing argument(s)\n");
+      Pike_error("Image.PNG.decode: missing argument(s)\n");
    
    image_png__decode(args);
    assign_svalue_no_free(&s,sp-1);
@@ -1674,13 +1694,8 @@ void exit_image_png(void)
      free_program(gz_deflate);
 }
 
-struct object *init_image_png(void)
+void init_image_png(void)
 {
-   struct program *p;
-   struct object *o;
-
-   start_new_program();
-
    push_text("Gz");
    push_int(0);
    SAFE_APPLY_MASTER("resolv",2);
@@ -1712,7 +1727,6 @@ struct object *init_image_png(void)
    }
    pop_stack();
 
-
    if (gz_deflate && gz_inflate && gz_crc32.type!=T_INT)
    {
       add_function("_chunk",image_png__chunk,
@@ -1722,10 +1736,14 @@ struct object *init_image_png(void)
 		   "function(string:array)",
 		   OPT_TRY_OPTIMIZE);
 
+      add_function("decode_header",image_png_decode_header,
+		   "function(string:mapping)",
+		   OPT_TRY_OPTIMIZE);
+
       if (gz_deflate)
       {
 	 add_function("_decode",image_png__decode,
-		      "function(array|string,void|mapping(string:mixed):object)",0);
+		      "function(array|string,void|mapping(string:mixed):mapping)",0);
 	 add_function("decode",image_png_decode,
 		      "function(string,void|mapping(string:mixed):object)",0);
 	 add_function("decode_alpha",image_png_decode_alpha,
@@ -1743,9 +1761,4 @@ struct object *init_image_png(void)
    param_bpp=make_shared_string("bpp");
    param_type=make_shared_string("type");
    param_background=make_shared_string("background");
-   
-   o = clone_object((p = end_program()),0);
-   free_program(p);
-
-   return o;
 }
