@@ -1,14 +1,16 @@
-/* $Id: font.c,v 1.41 1998/11/03 08:38:05 per Exp $ */
+/* $Id: font.c,v 1.68 2003/10/07 16:20:56 mast Exp $ */
 #include "global.h"
-#include <config.h>
 
 #define SPACE_CHAR 'i'
 
+extern unsigned char * image_default_font;
+#define IMAGE_DEFAULT_FONT_SIZE 30596
+
 /*
 **! module Image
 **! note
-**!	$Id: font.c,v 1.41 1998/11/03 08:38:05 per Exp $
-**! class font
+**!	$Id: font.c,v 1.68 2003/10/07 16:20:56 mast Exp $
+**! class Font
 **!
 **! note
 **! 	Short technical documentation on a font file:
@@ -88,7 +90,7 @@
 **!            	    len * (short char, short value)
 **!            **!	</pre>
 **!
-**! see also: Image, Image.image
+**! see also: Image, Image.Image
 */
 
 /* Dump a font into a Roxen Font file (format version 2)
@@ -138,9 +140,7 @@ Kerningtable types:
  */
 
 
-
-
-#include "fdlib.h"
+#include "image_machine.h"
 
 #include <sys/types.h>
 #include <sys/stat.h>
@@ -163,8 +163,6 @@ Kerningtable types:
 
 #include <errno.h>
 
-#include "config.h"
-
 #include "stralloc.h"
 #include "pike_macros.h"
 #include "object.h"
@@ -176,18 +174,28 @@ Kerningtable types:
 #include "builtin_functions.h"
 
 #include "image.h"
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #ifdef HAVE_MMAP
 #include <sys/mman.h>
 #endif
 
 #include "dmalloc.h"
 
-static struct program *font_program;
+#include "fdlib.h"
+
+#include "bignum.h"
+
+/* This must be included last! */
+#include "module_magic.h"
+
+extern struct program *font_program;
 extern struct program *image_program;
 
 #undef THIS
-#define THIS (*(struct font **)(fp->current_storage))
-#define THISOBJ (fp->current_object)
+#define THIS (*(struct font **)(Pike_fp->current_storage))
+#define THISOBJ (Pike_fp->current_object)
 
 struct font 
 {
@@ -198,8 +206,8 @@ struct font
 #endif
    void *mem;         /* pointer to mmaped/malloced memory */
    unsigned long chars;       /* number of characters */
-  float xspacing_scale; /* Fraction of spacing to use */
-  float yspacing_scale; /* Fraction of spacing to use */
+  double xspacing_scale; /* Fraction of spacing to use */
+  double yspacing_scale; /* Fraction of spacing to use */
   enum {
     J_LEFT,
     J_RIGHT,
@@ -219,7 +227,7 @@ static INLINE void free_font_struct(struct font *font)
 {
    if (font)
    {
-      if (font->mem)
+      if (font->mem && font->mem!=image_default_font)
       {
 #ifdef HAVE_MMAP
 	 munmap(font->mem,font->mmaped_size);
@@ -244,10 +252,10 @@ static void exit_font_struct(struct object *obj)
 
 /***************** internals ***********************************/
 
-static INLINE int char_space(struct font *this, INT32 c)
+static INLINE ptrdiff_t char_space(struct font *this, INT32 c)
 {
   if(c==0x20)
-    return (int)((float)(this->height*this->xspacing_scale)/4.5);
+    return DOUBLE_TO_INT((double)(this->height*this->xspacing_scale)/4.5);
   else if(c==0x20+128)
     return (this->height*this->xspacing_scale)/18;
   return this->charinfo[c].spacing*this->xspacing_scale;
@@ -260,9 +268,9 @@ static INLINE int char_width(struct font *this, INT32 c)
 }  
 
 #ifndef HAVE_MMAP
-static INLINE int my_read(int from, void *t, int towrite)
+static INLINE ptrdiff_t my_read(int from, void *t, size_t towrite)
 {
-  int res;
+  ptrdiff_t res;
   while((res = fd_read(from, t, towrite)) < 0)
   {
     switch(errno)
@@ -279,13 +287,12 @@ static INLINE int my_read(int from, void *t, int towrite)
 }
 #endif
 
-static INLINE long file_size(int fd)
+static INLINE off_t file_size(int fd)
 {
   struct stat tmp;
-  int res;
   if((!fd_fstat(fd, &tmp)) &&
-     (tmp.st_mode & S_IFREG)) 
-     return res = tmp.st_size;
+     ( tmp.st_mode & S_IFMT) == S_IFREG)
+     return (off_t)tmp.st_size;
   return -1;
 }
 
@@ -305,11 +312,12 @@ static INLINE void write_char(struct _char *ci,
       for (x=(INT32)ci->width; x>0; x--)
       {
 	 int r,c;
-	 if((c=255-*p)) 
+	 if((c=255-*p)) {
 	   if ((r=pos->r+c)>255)
 	     pos->r=pos->g=pos->b=255;
 	   else
 	     pos->r=pos->g=pos->b=r;
+	 }
 	 pos++;
 	 p++;
       }
@@ -336,49 +344,66 @@ void font_load(INT32 args);
 
 void font_create(INT32 args)
 {
-   if (args) 
-   {
-     font_load(args);
-     pop_stack();
-   }
+   font_load(args);
+   pop_stack();
 }
 
 void font_load(INT32 args)
 {
-   int fd;
+   int fd = -1;
+   size_t size;
 
-   if (args<1 
-       || sp[-args].type!=T_STRING)
-      error("font->read: illegal or wrong number of arguments\n");
-   
    if (THIS)
    {
       free_font_struct(THIS);
       THIS=NULL;
    }
+
+   if (!args) 
+   {
+      THIS=(struct font *)xalloc(sizeof(struct font));
+      THIS->mem=image_default_font;
+      size=IMAGE_DEFAULT_FONT_SIZE;
+      goto loading_default;
+   }
+
+   if (sp[-args].type!=T_STRING)
+      Pike_error("font->read: illegal or wrong number of arguments\n");
+   
    do 
    {
 #ifdef FONT_DEBUG
      fprintf(stderr,"FONT open '%s'\n",sp[-args].u.string->str);
 #endif
      fd = fd_open(sp[-args].u.string->str,fd_RDONLY,0);
+     /* FIXME: check_threads_etc(); ?
+      */
    } while(fd < 0 && errno == EINTR);
 
    if (fd >= 0)
    {
-      long size;
       struct font *new_font;
 
-      size = file_size(fd);
+      size = (size_t) file_size(fd);
       if (size > 0)
       {
-	 new_font=THIS=(struct font *)xalloc(sizeof(struct font));
+	 new_font=(struct font *)xalloc(sizeof(struct font));
 
 	 THREADS_ALLOW();
 #ifdef HAVE_MMAP
 	 new_font->mem = 
 	    mmap(0,size,PROT_READ,MAP_SHARED,fd,0);
-	 new_font->mmaped_size=size;
+#ifdef MAP_FAILED
+	 if ((char *)new_font->mem == (char *)MAP_FAILED)
+#else
+	 if (new_font->mem==(void*)-1)
+#endif
+	 {
+	    new_font->mem=0;
+	    new_font->mmaped_size=0;
+	 }
+	 else
+	    new_font->mmaped_size=size;
 #else
 	 new_font->mem = malloc(size);
 #ifdef FONT_DEBUG
@@ -392,6 +417,13 @@ void font_load(INT32 args)
 #endif
 	 THREADS_DISALLOW();
 
+	 if (THIS)
+	   /* In case font_load got called again in the THREADS_ALLOW block. */
+	   free_font_struct(THIS);
+	 THIS = new_font;
+
+loading_default:
+
 	 if (THIS->mem)
 	 {
 	    struct file_head 
@@ -449,8 +481,8 @@ void font_load(INT32 args)
 
 		  for (i=0; i<THIS->chars; i++)
 		  {
-		     if (i*sizeof(INT32)<(unsigned long)size
-			 && ntohl(fh->o[i])<(unsigned long)size
+		     if (i*sizeof(INT32)<(size_t)size
+			 && ntohl(fh->o[i])<(size_t)size
 			 && ! ( ntohl(fh->o[i]) % 4) ) /* must be aligned */
 		     {
 			ch=(struct char_head*)
@@ -467,6 +499,9 @@ void font_load(INT32 args)
 #endif
 			free_font_struct(new_font);
 			THIS=NULL;
+			if (fd >= 0) {
+			  fd_close(fd);
+			}
 			pop_n_elems(args);
 			push_int(0);
 			return;
@@ -474,6 +509,8 @@ void font_load(INT32 args)
 
 		  }
 
+		  if (!args) goto done;
+
 		  fd_close(fd);
 		  pop_n_elems(args);
 		  ref_push_object(THISOBJ);   /* success */
@@ -489,6 +526,7 @@ void font_load(INT32 args)
 #ifdef FONT_DEBUG
 	    else fprintf(stderr,"FONT wrong cookie\n");
 #endif
+	    if (!args) goto done; /* just in case */
 	 } /* mem failure */
 #ifdef FONT_DEBUG
 	 else fprintf(stderr,"FONT mem failure\n");
@@ -505,6 +543,8 @@ void font_load(INT32 args)
    else fprintf(stderr,"FONT fd failure\n");
 #endif
 
+done:
+
    pop_n_elems(args);
    push_int(0);
    return;
@@ -514,10 +554,10 @@ void font_load(INT32 args)
 **! method object write(string text,...)
 **! 	Writes some text; thus creating an image object
 **!	that can be used as mask or as a complete picture.
-**! returns an <ref>Image.image</ref> object
+**! returns an <ref>Image.Image</ref> object
 **! arg string text, ...
 **!	One or more lines of text.
-**! see also: text_extents, load, Image.image->paste_mask, Image.image->paste_alpha_color
+**! see also: text_extents, load, Image.Image->paste_mask, Image.Image->paste_alpha_color
 */
 
 void font_write(INT32 args)
@@ -529,11 +569,11 @@ void font_write(INT32 args)
    p_wchar0 *to_write0;
    p_wchar1 *to_write1;
    p_wchar2 *to_write2;
-   int to_write_len;
+   ptrdiff_t to_write_len;
    INT32 c;
-   struct font *this = (*(struct font **)(fp->current_storage));
+   struct font *this = (*(struct font **)(Pike_fp->current_storage));
    if (!this)
-      error("font->write: no font loaded\n");
+      Pike_error("font->write: no font loaded\n");
 
    if (args==0)
    {
@@ -544,13 +584,14 @@ void font_write(INT32 args)
    maxwidth2=1;
 
    width_of=(int *)malloc((args+1)*sizeof(int));
-   if(!width_of) error("Out of memory\n");
+   if(!width_of) resource_error(NULL,0,0,"memory",0,"Out of memory.\n");
 
    for (j=0; j<args; j++)
    {
       int max;
       if (sp[j-args].type!=T_STRING)
-	 error("font->write: illegal argument(s)\n");
+	 bad_arg_error("font->write",sp-args,args,0,"",sp-args,
+		"Bad arguments to font->write()\n");
      
       xsize = max = 1;
       to_write_len = sp[j-args].u.string->len;
@@ -606,13 +647,15 @@ void font_write(INT32 args)
    img = ((struct image*)o->storage);
    img->xsize = maxwidth2;
    if(args>1)
-     img->ysize = this->height+((double)this->height*(double)(args-1)*(double)this->yspacing_scale)+1;
+     img->ysize = DOUBLE_TO_INT(this->height+
+				((double)this->height*(double)(args-1)*
+				 (double)this->yspacing_scale)+1);
    else
      img->ysize = this->height;
    img->rgb.r=img->rgb.g=img->rgb.b=255;
    img->img=malloc(img->xsize*img->ysize*sizeof(rgb_group)+1);
 
-   if (!img) { free_object(o); free(width_of); error("Out of memory\n"); }
+   if (!img) { free_object(o); free(width_of); resource_error(NULL,0,0,"memory",0,"Out of memory.\n"); }
 
    MEMSET(img->img,0,img->xsize*img->ysize*sizeof(rgb_group));
 
@@ -631,7 +674,13 @@ void font_write(INT32 args)
      {
       case 0:
 	to_write0 = STR0(sp[j-args].u.string);
-	THREADS_ALLOW();
+	/* THREADS_ALLOW(); */
+	/* Can't release the thread here; this would point to freed
+	 * memory if font_load gets called. We'd have to refcount font
+	 * structs to cope with that, but since the released window
+	 * typically is short and since this code isn't used much now
+	 * when we have Image.FreeType, I bluntly disable it instead.
+	 * /mast */
 	for (i = 0; i < to_write_len; i++)
         {
 	  c=*(to_write0++);
@@ -640,17 +689,18 @@ void font_write(INT32 args)
 	    if(char_width(this,c))
 	      write_char(this->charinfo+c,
 			 (img->img+xsize)+
-			 (img->xsize*(int)(j*this->height*this->yspacing_scale)),
+			 (img->xsize*DOUBLE_TO_INT(j*this->height*
+						   this->yspacing_scale)),
 			 img->xsize,
 			 this->height);
 	    xsize += char_space(this, c);
 	  }
 	}
-	THREADS_DISALLOW();
+	/* THREADS_DISALLOW(); */
 	break;
       case 1:
 	to_write1 = STR1(sp[j-args].u.string);
-	THREADS_ALLOW();
+	/* THREADS_ALLOW(); */
 	for (i = 0; i < to_write_len; i++)
         {
 	  c=*(to_write1++);
@@ -659,17 +709,18 @@ void font_write(INT32 args)
 	    if(char_width(this,c))
 	      write_char(this->charinfo+c,
 			 (img->img+xsize)+
-			 (img->xsize*(int)(j*this->height*this->yspacing_scale)),
+			 (img->xsize*DOUBLE_TO_INT(j*this->height*
+						   this->yspacing_scale)),
 			 img->xsize,
 			 this->height);
 	    xsize += char_space(this, c);
 	  }
 	}
-	THREADS_DISALLOW();
+	/* THREADS_DISALLOW(); */
 	break;
       case 2:
 	to_write2 = STR2(sp[j-args].u.string);
-	THREADS_ALLOW();
+	/* THREADS_ALLOW(); */
 	for (i = 0; i < to_write_len; i++)
         {
 	  c=*(to_write2++);
@@ -678,13 +729,14 @@ void font_write(INT32 args)
 	    if(char_width(this,c))
 	      write_char(this->charinfo+c,
 			 (img->img+xsize)+
-			 (img->xsize*(int)(j*this->height*this->yspacing_scale)),
+			 (img->xsize*DOUBLE_TO_INT(j*this->height*
+						   this->yspacing_scale)),
 			 img->xsize,
 			 this->height);
 	    xsize += char_space(this, c);
 	  }
 	}
-	THREADS_DISALLOW();
+	/* THREADS_DISALLOW(); */
 	break;
       default:
 	fatal("Illegal shift size!\n");
@@ -726,7 +778,7 @@ void font_text_extents(INT32 args)
 {
   INT32 xsize,i,maxwidth2,j;
 
-  if (!THIS) error("font->text_extents: no font loaded\n");
+  if (!THIS) Pike_error("font->text_extents: no font loaded\n");
 
   maxwidth2=0;
 
@@ -742,9 +794,10 @@ void font_text_extents(INT32 args)
      p_wchar0 *to_write0;
      p_wchar1 *to_write1;
      p_wchar2 *to_write2;
-     int to_write_len;
+     ptrdiff_t to_write_len;
      if (sp[j-args].type!=T_STRING)
-	error("font->write: illegal argument(s)\n");
+	bad_arg_error("font->text_extents",sp-args,args,0,"",sp-args,
+		"Bad arguments to font->text_extents()\n");
      
      xsize = max = 1;
      to_write_len = sp[j-args].u.string->len;
@@ -788,7 +841,7 @@ void font_text_extents(INT32 args)
 
   pop_n_elems(args);
   push_int(maxwidth2);
-  push_int(args * THIS->height * THIS->yspacing_scale);
+  push_int64(args * THIS->height * THIS->yspacing_scale);
   f_aggregate(2);
 }
 
@@ -806,10 +859,10 @@ void font_text_extents(INT32 args)
 
 void font_set_xspacing_scale(INT32 args)
 {
-  if(!THIS) error("font->set_xspacing_scale(FLOAT): No font loaded.\n");
-  if(!args) error("font->set_xspacing_scale(FLOAT): No argument!\n");
+  if(!THIS) Pike_error("font->set_xspacing_scale(FLOAT): No font loaded.\n");
+  if(!args) Pike_error("font->set_xspacing_scale(FLOAT): No argument!\n");
   if(sp[-args].type!=T_FLOAT)
-    error("font->set_xspacing_scale(FLOAT): Wrong type of argument!\n");
+    Pike_error("font->set_xspacing_scale(FLOAT): Wrong type of argument!\n");
 
   THIS->xspacing_scale = (double)sp[-args].u.float_number;
 /*fprintf(stderr, "Setting xspacing to %f\n", THIS->xspacing_scale);*/
@@ -821,10 +874,10 @@ void font_set_xspacing_scale(INT32 args)
 
 void font_set_yspacing_scale(INT32 args)
 {
-  if(!THIS) error("font->set_yspacing_scale(FLOAT): No font loaded.\n");
-  if(!args) error("font->set_yspacing_scale(FLOAT): No argument!\n");
+  if(!THIS) Pike_error("font->set_yspacing_scale(FLOAT): No font loaded.\n");
+  if(!args) Pike_error("font->set_yspacing_scale(FLOAT): No argument!\n");
   if(sp[-args].type!=T_FLOAT)
-    error("font->set_yspacing_scale(FLOAT): Wrong type of argument!\n");
+    Pike_error("font->set_yspacing_scale(FLOAT): Wrong type of argument!\n");
 
   THIS->yspacing_scale = (double)sp[-args].u.float_number;
 /*fprintf(stderr, "Setting yspacing to %f\n", THIS->yspacing_scale);*/
@@ -878,57 +931,51 @@ int baseline();             // font baseline
 
 */
 
-void init_font_programs(void)
+void init_image_font(void)
 {
-   start_new_program();
-   add_storage(sizeof(struct font*));
+   ADD_STORAGE(struct font*);
 
-   add_function("load",font_load,
-                "function(string:object|int)",0);
+   /* function(string:object|int) */
+   ADD_FUNCTION("load",font_load,tFunc(tStr,tOr(tObj,tInt)),0);
 
-   add_function("create",font_create,
-                "function(void|string:void)",0);
+   /* function(void|string:void) */
+   ADD_FUNCTION("create",font_create,tFunc(tOr(tVoid,tStr),tVoid),0);
 
-   add_function("write",font_write,
-                "function(string:object)",0);
+   /* function(string:object) */
+   ADD_FUNCTION("write",font_write,tFuncV(tNone,tStr,tObj),0);
 
-   add_function("height",font_height,
-                "function(:int)",0);
+   /* function(:int) */
+   ADD_FUNCTION("height",font_height,tFunc(tNone,tInt),0);
 
-   add_function("baseline",font_baseline,
-                "function(:int)",0);
+   /* function(:int) */
+   ADD_FUNCTION("baseline",font_baseline,tFunc(tNone,tInt),0);
 		
-   add_function("extents",font_text_extents,
-                "function(string ...:array(int))",0);
+   /* function(string ...:array(int)) */
+   ADD_FUNCTION("extents",font_text_extents,tFuncV(tNone,tStr,tArr(tInt)),0);
 		
-   add_function("text_extents",font_text_extents,
-                "function(string ...:array(int))",0);
+   /* function(string ...:array(int)) */
+   ADD_FUNCTION("text_extents",font_text_extents,tFuncV(tNone,tStr,tArr(tInt)),0);
 		
-   add_function("set_x_spacing",font_set_xspacing_scale,
-                "function(float:void)",0);
+   /* function(float:void) */
+   ADD_FUNCTION("set_x_spacing",font_set_xspacing_scale,tFunc(tFlt,tVoid),0);
 
-   add_function("set_y_spacing",font_set_yspacing_scale,
-                "function(float:void)",0);
+   /* function(float:void) */
+   ADD_FUNCTION("set_y_spacing",font_set_yspacing_scale,tFunc(tFlt,tVoid),0);
 
-   add_function("center", font_set_center, "function(void:void)", 0);
-   add_function("left", font_set_left, "function(void:void)", 0);
-   add_function("right", font_set_right, "function(void:void)", 0);
+   /* function(void:void) */
+   ADD_FUNCTION("center", font_set_center,tFunc(tVoid,tVoid), 0);
+   /* function(void:void) */
+   ADD_FUNCTION("left", font_set_left,tFunc(tVoid,tVoid), 0);
+   /* function(void:void) */
+   ADD_FUNCTION("right", font_set_right,tFunc(tVoid,tVoid), 0);
 
    
    set_init_callback(init_font_struct);
    set_exit_callback(exit_font_struct);
-  
-   font_program=end_program();
-   add_program_constant("font",font_program, 0);
 }
 
-void exit_font(void) 
+void exit_image_font(void) 
 {
-  if(font_program)
-  {
-    free_program(font_program);
-    font_program=0;
-  }
 }
 
 
