@@ -1,9 +1,9 @@
-/* $Id: operator.c,v 1.37 2000/12/27 19:27:34 per Exp $ */
+/* $Id: operator.c,v 1.39 2004/05/15 19:19:24 jonasw Exp $ */
 
 /*
 **! module Image
 **! note
-**!	$Id: operator.c,v 1.37 2000/12/27 19:27:34 per Exp $
+**!	$Id: operator.c,v 1.39 2004/05/15 19:19:24 jonasw Exp $
 **! class Image
 */
 
@@ -172,9 +172,9 @@ STANDARD_OPERATOR_HEADER("`+")
        image_add_buffers_mmx_x86asm( d, s1, s2, (i*3)/8 );
        nleft = (i*3)%8;
        for( ; nleft; nleft-- )
-         ((unsigned char *)d)[i-nleft-1] = 
-                    MINIMUM((((unsigned char *)s1)[i-nleft-1] +
-                             ((unsigned char *)s2)[i-nleft-1]), 255 );
+         ((unsigned char *)d)[i * 3 - nleft] = 
+                    MINIMUM((((unsigned char *)s1)[i * 3 - nleft] +
+                             ((unsigned char *)s2)[i * 3 - nleft]), 255 );
      } else 
 #endif
      while (i--)
@@ -253,6 +253,7 @@ void image_operator_multiply(INT32 args)
    double q=1/255.0;
 STANDARD_OPERATOR_HEADER("`*")
   {
+#if 0
 #ifdef ASSEMBLY_OK
      if( image_cpuid & IMAGE_MMX )
      {
@@ -264,6 +265,7 @@ STANDARD_OPERATOR_HEADER("`*")
                     (((unsigned char *)s1)[i-nleft-1] * 
                      ((unsigned char *)s2)[i-nleft-1]) / 255;
      } else
+#endif
 #endif
      while (i--)
      {
@@ -277,6 +279,7 @@ STANDARD_OPERATOR_HEADER("`*")
             (rgb.g < 256) &&
             (rgb.b < 256) )
    {
+#if 0
 #ifdef ASSEMBLY_OK
      /* there is some overhead in setting this up */
      if( (image_cpuid & IMAGE_MMX) && (i>40) )
@@ -286,6 +289,7 @@ STANDARD_OPERATOR_HEADER("`*")
        i = i%4;
        d -= i;  s1 -= i;
      }
+#endif
 #endif
      while (i--)
      {
