@@ -24,9 +24,13 @@
 #endif
 #endif
 
+#ifdef HAVE_SYS_ID_H
+#include <sys/id.h>
+#endif /* HAVE_SYS_ID_H */
+
 #include <fcntl.h>
 
-RCSID("$Id: pipe.c,v 1.28 1998/10/23 01:03:18 grubba Exp $");
+RCSID("$Id: pipe.c,v 1.46 2003/10/07 16:27:19 mast Exp $");
 
 #include "threads.h"
 #include "stralloc.h"
@@ -35,8 +39,9 @@ RCSID("$Id: pipe.c,v 1.28 1998/10/23 01:03:18 grubba Exp $");
 #include "constants.h"
 #include "interpret.h"
 #include "svalue.h"
-#include "error.h"
+#include "pike_error.h"
 #include "builtin_functions.h"
+#include "fdlib.h"
 
 #ifndef S_ISREG
 #ifdef S_IFREG
@@ -46,6 +51,10 @@ RCSID("$Id: pipe.c,v 1.28 1998/10/23 01:03:18 grubba Exp $");
 #endif
 #endif
 
+/* must be included last */
+#include "module_magic.h"
+
+
 /*
 #define PIPE_STRING_DEBUG
 #define PIPE_MMAP_DEBUG
@@ -101,8 +110,8 @@ static struct program *pipe_program, *output_program;
 #ifdef THIS
 #undef THIS
 #endif
-#define THIS ((struct pipe *)(fp->current_storage))
-#define THISOBJ (fp->current_object)
+#define THIS ((struct pipe *)(Pike_fp->current_storage))
+#define THISOBJ (Pike_fp->current_object)
 
 struct input
 {
@@ -113,15 +122,15 @@ struct input
     struct pike_string *str;
     char *mmap;
   } u;
-  unsigned long len;		/* current input: string or mmap len */
-  int set_blocking_offset, set_nonblocking_offset;
+  size_t len;		/* current input: string or mmap len */
+  ptrdiff_t set_blocking_offset, set_nonblocking_offset;
   struct input *next;
 };
 
 struct output
 {
   struct object *obj;
-  int write_offset, set_blocking_offset, set_nonblocking_offset;
+  ptrdiff_t write_offset, set_blocking_offset, set_nonblocking_offset;
   int fd;
       
   enum 
@@ -130,7 +139,7 @@ struct output
     O_SLEEP			/* sleeping; waiting for more data */
   } mode;
 
-  unsigned long pos; /* position in buffer */
+  size_t pos; /* position in buffer */
   struct object *next;
   struct pipe *the_pipe;
 };
@@ -158,7 +167,7 @@ struct pipe
   int fd;			/* buffer fd or -1 */
 
   unsigned long bytes_in_buffer;
-  unsigned long pos; 
+  size_t pos; 
   /* fd: size of buffer file */
   /* current position of first element (buffer or mmap) */
   struct buffer *firstbuffer,*lastbuffer;
@@ -169,11 +178,12 @@ struct pipe
   unsigned long sent;
 };
 
-static int offset_input_read_callback;
-static int offset_input_close_callback;
-static int offset_output_write_callback;
-static int offset_output_close_callback;
-static int mmapped, nobjects, nstrings, noutputs, ninputs, nbuffers, sbuffers;
+static ptrdiff_t offset_input_read_callback;
+static ptrdiff_t offset_input_close_callback;
+static ptrdiff_t offset_output_write_callback;
+static ptrdiff_t offset_output_close_callback;
+static ptrdiff_t mmapped, nobjects, nstrings, noutputs;
+static ptrdiff_t ninputs, nbuffers, sbuffers;
 
 void close_and_free_everything(struct object *o,struct pipe *);
 static INLINE void output_finish(struct object *obj);
@@ -183,12 +193,12 @@ static INLINE void output_try_write_some(struct object *obj);
 
 /* Push a callback to this object given the internal function number.
  */
-static void push_callback(int no)
+static void push_callback(ptrdiff_t no)
 {
-  add_ref(sp->u.object=THISOBJ);
-  sp->subtype=no+fp->context.identifier_level;
-  sp->type=T_FUNCTION;
-  sp++;
+  add_ref(Pike_sp->u.object=THISOBJ);
+  Pike_sp->subtype = DO_NOT_WARN(no + Pike_fp->context.identifier_level);
+  Pike_sp->type = T_FUNCTION;
+  Pike_sp++;
 }
 
 /* Allocate a new struct input, link it last in the linked list */
@@ -210,6 +220,8 @@ static INLINE struct input *new_input(void)
 /* Free an input struct and all that it stands for */
 static INLINE void free_input(struct input *i)
 {
+  debug_malloc_touch(i);
+
   ninputs--;
   switch (i->type)
   {
@@ -241,7 +253,7 @@ static INLINE void free_input(struct input *i)
     munmap(i->u.mmap,i->len);
     mmapped -= i->len;  
 #else
-    error("I_MMAP input when MMAP is diabled!");
+    Pike_error("I_MMAP input when MMAP is diabled!");
 #endif
     break;
 
@@ -261,7 +273,7 @@ static INLINE void pipe_done(void)
 
     if(!THISOBJ->prog) /* We will not free anything in this case. */
       return;
-    /*  error("Pipe done callback destructed pipe.\n");  */
+    /*  Pike_error("Pipe done callback destructed pipe.\n");  */
   }
   close_and_free_everything(THISOBJ,THIS);
 }
@@ -290,10 +302,12 @@ static INLINE int append_buffer(struct pike_string *s)
 {
    struct buffer *b;
 
+   debug_malloc_touch(s);
+
    if(THIS->fd!= -1)
    {
-     lseek(THIS->fd, THIS->pos, SEEK_SET);
-     write(THIS->fd, s->str, s->len);
+     fd_lseek(THIS->fd, THIS->pos, SEEK_SET);
+     fd_write(THIS->fd, s->str, s->len);
      THIS->pos+=s->len;
      return 0;
    }
@@ -328,8 +342,9 @@ static void low_start(void)
   add_ref(THISOBJ);		/* dont kill yourself now */
   for(obj=THIS->firstoutput;obj;obj=next)
   {
-    add_ref(obj);		/* Hang on PLEASE!! /hubbe */
+    /* add_ref(obj); */		/* Hang on PLEASE!! /hubbe */
     o=(struct output *)(obj->storage);
+    next=o->next;
     if (o->obj && o->mode==O_SLEEP)
     {
       if (!o->obj->prog)
@@ -338,18 +353,16 @@ static void low_start(void)
       }
       else
       {
-#if 0
 	push_int(0);
 	push_callback(offset_output_write_callback);
 	push_callback(offset_output_close_callback);
 	apply_low(o->obj,o->set_nonblocking_offset,3);
-#endif
-	output_try_write_some(obj);
-	o->mode=O_RUN;		/* Hubbe */
+	/* output_try_write_some(obj); */
+	/* o->mode=O_RUN; */		/* Hubbe */
       }
     }
-    next=o->next;
-    free_object(obj);
+    /* next=o->next; */
+    /* free_object(obj); */
   }
 
   free_object(THISOBJ);
@@ -434,10 +447,10 @@ static INLINE void input_finish(void)
 /* This function reads some data from the file cache..
  * Called when we want some data to send.
  */
-static INLINE struct pike_string* gimme_some_data(unsigned long pos)
+static INLINE struct pike_string* gimme_some_data(size_t pos)
 {
    struct buffer *b;
-   long len;
+   ptrdiff_t len;
    struct pipe *this = THIS;
 
    /* We have a file cache, read from it */
@@ -449,11 +462,11 @@ static INLINE struct pike_string* gimme_some_data(unsigned long pos)
       len=this->pos-pos;
       if (len>READ_BUFFER_SIZE) len=READ_BUFFER_SIZE;
       THREADS_ALLOW();
-      lseek(this->fd, pos, SEEK_SET);
+      fd_lseek(this->fd, pos, SEEK_SET);
       THREADS_DISALLOW();
       do {
 	THREADS_ALLOW();
-	len = read(this->fd, buffer, len);
+	len = fd_read(this->fd, buffer, len);
 	THREADS_DISALLOW();
 	if (len < 0) {
 	  if (errno != EINTR) {
@@ -564,12 +577,26 @@ static INLINE struct pike_string* gimme_some_data(unsigned long pos)
  */
 static INLINE void output_finish(struct object *obj)
 {
-  struct output *o;
+  struct output *o, *oi;
+  struct object *obji;
+
+  debug_malloc_touch(obj);
 
   o=(struct output *)(obj->storage);
 
   if (o->obj)
   {
+    if(obj==THIS->firstoutput){
+      THIS->firstoutput=o->next;
+    } else
+    for(obji=THIS->firstoutput;obji;obji=oi->next)
+    {
+      oi=(struct output *)(obji->storage);
+      if(oi->next==obj)
+      {
+       oi->next=o->next;
+      }
+    }
     if(o->obj->prog)
     {
 #ifdef BLOCKING_CLOSE
@@ -583,7 +610,7 @@ static INLINE void output_finish(struct object *obj)
       apply(o->obj,"close",0);
       pop_stack();
       if(!THISOBJ->prog)
-	error("Pipe done callback destructed pipe.\n");
+	Pike_error("Pipe done callback destructed pipe.\n");
       destruct(o->obj);
     }
     free_object(o->obj);
@@ -594,7 +621,7 @@ static INLINE void output_finish(struct object *obj)
 
     finished_p(); /* Moved by per, one line down.. :) */
 
-    free_object(THISOBJ);		/* What? /Hubbe */
+    /* free_object(THISOBJ); */		/* What? /Hubbe */
   }
 }
 
@@ -605,9 +632,11 @@ static INLINE void output_try_write_some(struct object *obj)
 {
   struct output *out;
   struct pike_string *s;
-  unsigned long len;
-  INT32 ret;
+  size_t len;
+  INT_TYPE ret;
   
+   debug_malloc_touch(obj);
+
   out=(struct output*)(obj->storage);
 
 #ifdef INSISTANT_WRITE   
@@ -625,10 +654,8 @@ static INLINE void output_try_write_some(struct object *obj)
       }
       else
       {
-#if 0
 	apply_low(out->obj, out->set_blocking_offset, 0);
 	pop_stack();		/* from apply */
-#endif
 	out->mode=O_SLEEP;
       }
       return;
@@ -667,11 +694,11 @@ static void pipe_input(INT32 args)
    struct object *obj;
 
    if (args<1 || sp[-args].type != T_OBJECT)
-     error("Bad/missing argument 1 to pipe->input().\n");
+     Pike_error("Bad/missing argument 1 to pipe->input().\n");
 
    obj=sp[-args].u.object;
    if(!obj || !obj->prog)
-     error("pipe->input() on destructed object.\n");
+     Pike_error("pipe->input() on destructed object.\n");
 
    push_int(0);
    apply(sp[-args-1].u.object,"set_id", 1);
@@ -690,20 +717,21 @@ static void pipe_input(INT32 args)
 
      if (fd != -1 && fstat(fd,&s)==0)
      {
-       int filep=lseek(fd, 0L, SEEK_CUR); /* keep the file pointer */
+       off_t filep=fd_lseek(fd, 0L, SEEK_CUR); /* keep the file pointer */
+       size_t len = s.st_size - filep;
        if(S_ISREG(s.st_mode)	/* regular file */
-	  && ((long)(m=(char *)mmap(0,s.st_size - filep,PROT_READ,
+	  && ((long)(m=(char *)mmap(0, len, PROT_READ,
 				    MAP_FILE|MAP_SHARED,fd,filep))!=-1))
        {
 #ifdef HAVE_GETEUID
 	 int ou = 0;
 #endif
-	 mmapped += s.st_size;
+	 mmapped += len;
 
 	 i->type=I_MMAP;
-	 i->len=s.st_size;
+	 i->len = len;
 	 i->u.mmap=m;
-#ifdef HAVE_MADVISE
+#if defined(HAVE_MADVISE) && defined(MADV_SEQUENTIAL)
 	 /* Mark the pages as sequential read only access... */
 
 	 /* NOTE:
@@ -722,7 +750,7 @@ static void pipe_input(INT32 args)
 #endif /* HAVE_SETEUID */
 	 }
 #endif
-	 madvise(m, s.st_size, MADV_SEQUENTIAL);
+	 madvise(m, len, MADV_SEQUENTIAL);
 #ifdef HAVE_GETEUID
 	 if(ou) {
 #ifdef HAVE_SETEUID
@@ -760,7 +788,7 @@ static void pipe_input(INT32 args)
 	 i->type=I_NONE;
 
 	 nobjects--;
-	 error("illegal file object%s%s\n",
+	 Pike_error("illegal file object%s%s\n",
 	       ((i->set_nonblocking_offset<0)?"; no set_nonblocking":""),
 	       ((i->set_blocking_offset<0)?"; no set_blocking":""));
       } else {
@@ -803,7 +831,7 @@ static void pipe_write(INT32 args)
   struct input *i;
 
   if (args<1 || sp[-args].type!=T_STRING)
-    error("illegal argument to pipe->write()\n");
+    Pike_error("illegal argument to pipe->write()\n");
 
   if (!THIS->firstinput)
   {
@@ -834,8 +862,12 @@ static void pipe_output(INT32 args)
       sp[-args].type != T_OBJECT ||
       !sp[-args].u.object ||
       !sp[-args].u.object->prog)
-    error("Bad/missing argument 1 to pipe->output().\n");
+    Pike_error("Bad/missing argument 1 to pipe->output().\n");
 
+  if (args==2 &&
+      sp[1-args].type != T_INT)
+    Pike_error("Bad argument 2 to pipe->output().\n");
+       
   if (THIS->fd==-1)		/* no buffer */
   {
     /* test if usable as buffer */ 
@@ -845,10 +877,10 @@ static void pipe_output(INT32 args)
 	&& (fd=sp[-1].u.integer)>=0
 	&& (fstat(fd,&s)==0)
 	&& S_ISREG(s.st_mode)
-	&& (THIS->fd=dup(fd))!=-1 )
+	&& (THIS->fd=fd_dup(fd))!=-1 )
     {
       /* keep the file pointer of the duped fd */
-      THIS->pos=lseek(fd, 0L, SEEK_CUR);
+      THIS->pos=fd_lseek(fd, 0L, SEEK_CUR);
       
 #if 0
       /* This won't work if the spider-module is dynamically linked. */
@@ -864,8 +896,8 @@ static void pipe_output(INT32 args)
       {
 	b=THIS->firstbuffer;
 	THIS->firstbuffer=b->next;
-	lseek(THIS->fd, THIS->pos, SEEK_SET);
-	write(THIS->fd,b->s->str,b->s->len);
+	fd_lseek(THIS->fd, THIS->pos, SEEK_SET);
+	fd_write(THIS->fd,b->s->str,b->s->len);
 	sbuffers-=b->s->len;
 	nbuffers--;
 	free_string(b->s);
@@ -884,7 +916,7 @@ static void pipe_output(INT32 args)
   } 
 
   THIS->living_outputs++;
-  add_ref(THISOBJ);		/* Weird */
+  /* add_ref(THISOBJ); */	/* Weird */
 
   /* Allocate a new struct output */
   obj=clone_object(output_program,0);
@@ -904,7 +936,7 @@ static void pipe_output(INT32 args)
       o->set_blocking_offset<0) 
   {
     free_object(o->obj);
-    error("illegal file object%s%s%s\n",
+    Pike_error("illegal file object%s%s%s\n",
 	  ((o->write_offset<0)?"; no write":""),
 	  ((o->set_nonblocking_offset<0)?"; no set_nonblocking":""),
 	  ((o->set_blocking_offset<0)?"; no set_blocking":""));
@@ -913,9 +945,15 @@ static void pipe_output(INT32 args)
   o->mode=O_RUN;
   /* keep the file pointer of the duped fd
      o->pos=0; */
-  o->pos=THIS->pos;
+  /* allow start position as 2nd argument for additional outputs
+  o->pos=THIS->pos; */
 
-  ref_push_object(obj);
+  if(args>=2)
+    o->pos=sp[1-args].u.integer;
+  else
+    o->pos=THIS->pos;
+
+  push_object(obj); /* Ok, David, this is probably correct, but I dare you to explain why :) */
   apply(o->obj,"set_id",1);
   pop_stack();
 
@@ -937,7 +975,7 @@ static void pipe_set_done_callback(INT32 args)
     return;
   }
   if (args<1 || (sp[-args].type!=T_FUNCTION && sp[-args].type!=T_ARRAY))
-    error("Illegal argument to set_done_callback()\n");
+    Pike_error("Illegal argument to set_done_callback()\n");
 
   if (args>1)
   {
@@ -954,12 +992,12 @@ static void pipe_set_output_closed_callback(INT32 args)
 {
   if (args==0)
   {
-    free_svalue(&THIS->done_callback);
+    free_svalue(&THIS->output_closed_callback);
     THIS->output_closed_callback.type=T_INT;
     return;
   }
   if (args<1 || (sp[-args].type!=T_FUNCTION && sp[-args].type!=T_ARRAY))
-    error("Illegal argument to set_output_closed_callback()\n");
+    Pike_error("Illegal argument to set_output_closed_callback()\n");
 
   if (args>1)
   {
@@ -996,13 +1034,14 @@ static void f_bytes_sent(INT32 args)
 static void pipe_write_output_callback(INT32 args)
 {
    if (args<1 || sp[-args].type!=T_OBJECT)
-     error("Illegal argument to pipe->write_output_callback\n");
+     Pike_error("Illegal argument to pipe->write_output_callback\n");
 
    if(!sp[-args].u.object->prog) return;
 
    if(sp[-args].u.object->prog != output_program)
-     error("Illegal argument to pipe->write_output_callback\n");
+     Pike_error("Illegal argument to pipe->write_output_callback\n");
 
+   debug_malloc_touch(sp[-args].u.object);
    output_try_write_some(sp[-args].u.object);
    pop_n_elems(args-1);
 }
@@ -1015,7 +1054,7 @@ static void pipe_close_output_callback(INT32 args)
    if(!sp[-args].u.object->prog) return;
 
    if(sp[-args].u.object->prog != output_program)
-     error("Illegal argument to pipe->close_output_callback\n");
+     Pike_error("Illegal argument to pipe->close_output_callback\n");
 
   o=(struct output *)(sp[-args].u.object->storage);
 
@@ -1037,12 +1076,12 @@ static void pipe_read_input_callback(INT32 args)
   struct pike_string *s;
 
   if (args<2 || sp[1-args].type!=T_STRING)
-    error("Illegal argument to pipe->read_input_callback\n");
+    Pike_error("Illegal argument to pipe->read_input_callback\n");
    
   i=THIS->firstinput;
 
   if (!i)
-    error("Pipe read callback without any inputs left.\n");
+    Pike_error("Pipe read callback without any inputs left.\n");
 
   s=sp[1-args].u.string;
 
@@ -1067,10 +1106,10 @@ static void pipe_close_input_callback(INT32 args)
    i=THIS->firstinput;
 
    if(!i)
-     error("Input close callback without inputs!\n");
+     Pike_error("Input close callback without inputs!\n");
 
    if(i->type != I_OBJ)
-     error("Premature close callback on pipe!.\n");
+     Pike_error("Premature close callback on pipe!.\n");
 
    if (i->u.obj->prog)
    {
@@ -1104,7 +1143,13 @@ void close_and_free_everything(struct object *thisobj,struct pipe *p)
    struct input *i;
    struct output *o;
    struct object *obj;
+
+   debug_malloc_touch(thisobj);
+   debug_malloc_touch(p);
    
+   if(p->done){
+     return;
+   }
    p->done=1;
 
    if (thisobj) 
@@ -1141,7 +1186,7 @@ void close_and_free_everything(struct object *thisobj,struct pipe *p)
    }
    if (p->fd!=-1)
    {
-     close(p->fd);
+     fd_close(p->fd);
      p->fd=-1;
    }
 
@@ -1158,11 +1203,13 @@ void close_and_free_everything(struct object *thisobj,struct pipe *p)
    p->output_closed_callback.type=T_INT;
    p->id.type=T_INT;
 
-   p->done=0;
+   /* p->done=0; */
 }
 
 static void init_pipe_struct(struct object *o)
 {
+   debug_malloc_touch(o);
+
    THIS->firstbuffer=THIS->lastbuffer=NULL;
    THIS->firstinput=THIS->lastinput=NULL;
    THIS->firstoutput=NULL;
@@ -1188,7 +1235,9 @@ static void exit_output_struct(struct object *obj)
 {
   struct output *o;
   
-  o=(struct output *)(fp->current_storage);
+   debug_malloc_touch(obj);
+  o=(struct output *)(Pike_fp->current_storage);
+
   if (o->obj)
   {
     if(o->obj->prog)
@@ -1205,18 +1254,20 @@ static void exit_output_struct(struct object *obj)
       pop_stack();
 
       if(!THISOBJ->prog)
-	error("Pipe done callback destructed pipe.\n");
+	Pike_error("Pipe done callback destructed pipe.\n");
     }
     free_object(o->obj);
     noutputs--;
     o->obj=0;
+    o->fd=-1;
   }
 }
 
 static void init_output_struct(struct object *ob)
 {
   struct output *o;
-  o=(struct output *)(fp->current_storage);
+  debug_malloc_touch(ob);
+  o=(struct output *)(Pike_fp->current_storage);
   o->obj=0;
 }
 
@@ -1228,45 +1279,54 @@ void port_setup_program(void);
 void f__pipe_debug(INT32 args)
 {
   pop_n_elems(args);
-  push_int(noutputs);
-  push_int(ninputs);
-  push_int(nstrings);
-  push_int(nobjects);
-  push_int(mmapped);
-  push_int(nbuffers);
-  push_int(sbuffers);
+  push_int(DO_NOT_WARN(noutputs));
+  push_int(DO_NOT_WARN(ninputs));
+  push_int(DO_NOT_WARN(nstrings));
+  push_int(DO_NOT_WARN(nobjects));
+  push_int(DO_NOT_WARN(mmapped));
+  push_int(DO_NOT_WARN(nbuffers));
+  push_int(DO_NOT_WARN(sbuffers));
   f_aggregate(7);
 }
 
 void pike_module_init(void)
 {
    start_new_program();
-   add_storage(sizeof(struct pipe));
-   add_efun("_pipe_debug", f__pipe_debug, "function(:array)", 0);
-   add_function("input",pipe_input,"function(object:void)",0);
-   add_function("output",pipe_output,"function(object:void)",0);
-   add_function("write",pipe_write,"function(string:void)",0);
+   ADD_STORAGE(struct pipe);
+   
+/* function(:array) */
+  ADD_EFUN("_pipe_debug", f__pipe_debug,tFunc(tNone,tArray), 0);
+   /* function(object:void) */
+  ADD_FUNCTION("input",pipe_input,tFunc(tObj,tVoid),0);
+   /* function(object,void|int:void) */
+  ADD_FUNCTION("output",pipe_output,tFunc(tObj tOr(tVoid,tInt),tVoid),0);
+   /* function(string:void) */
+  ADD_FUNCTION("write",pipe_write,tFunc(tStr,tVoid),0);
 
-   add_function("start",pipe_start,"function(:void)",0);
-   add_function("finish",pipe_finish,"function(:void)",0);
+   /* function(:void) */
+  ADD_FUNCTION("start",pipe_start,tFunc(tNone,tVoid),0);
+   /* function(:void) */
+  ADD_FUNCTION("finish",pipe_finish,tFunc(tNone,tVoid),0);
    
-   add_function("set_output_closed_callback",pipe_set_output_closed_callback,
-		"function(void|function(mixed,object:mixed),void|mixed:void)",0);
-   add_function("set_done_callback",pipe_set_done_callback,
-		"function(void|function(mixed:mixed),void|mixed:void)",0);
+   /* function(void|function(mixed,object:mixed),void|mixed:void) */
+  ADD_FUNCTION("set_output_closed_callback",pipe_set_output_closed_callback,tFunc(tOr(tVoid,tFunc(tMix tObj,tMix)) tOr(tVoid,tMix),tVoid),0);
+   /* function(void|function(mixed:mixed),void|mixed:void) */
+  ADD_FUNCTION("set_done_callback",pipe_set_done_callback,tFunc(tOr(tVoid,tFunc(tMix,tMix)) tOr(tVoid,tMix),tVoid),0);
    
-   add_function("_output_close_callback",pipe_close_output_callback,
-		"function(int:void)",0);
-   add_function("_input_close_callback",pipe_close_input_callback,
-		"function(int:void)",0);
-   add_function("_output_write_callback",pipe_write_output_callback,
-		"function(int:void)",0);
-   add_function("_input_read_callback",pipe_read_input_callback,
-		"function(int,string:void)",0);
+   /* function(int:void) */
+  ADD_FUNCTION("_output_close_callback",pipe_close_output_callback,tFunc(tInt,tVoid),0);
+   /* function(int:void) */
+  ADD_FUNCTION("_input_close_callback",pipe_close_input_callback,tFunc(tInt,tVoid),0);
+   /* function(int:void) */
+  ADD_FUNCTION("_output_write_callback",pipe_write_output_callback,tFunc(tInt,tVoid),0);
+   /* function(int,string:void) */
+  ADD_FUNCTION("_input_read_callback",pipe_read_input_callback,tFunc(tInt tStr,tVoid),0);
 
-   add_function("version",pipe_version,"function(:string)",0);
+   /* function(:string) */
+  ADD_FUNCTION("version",pipe_version,tFunc(tNone,tStr),0);
    
-   add_function("bytes_sent",f_bytes_sent,"function(:int)",0);
+   /* function(:int) */
+  ADD_FUNCTION("bytes_sent",f_bytes_sent,tFunc(tNone,tInt),0);
 
    set_init_callback(init_pipe_struct);
    set_exit_callback(exit_pipe_struct);
@@ -1285,7 +1345,7 @@ void pike_module_init(void)
 
 
    start_new_program();
-   add_storage(sizeof(struct output));
+   ADD_STORAGE(struct output);
    set_init_callback(init_output_struct);
    set_exit_callback(exit_output_struct);
    output_program=end_program();
