@@ -40,6 +40,7 @@
     @  do this format for each entry in argument array
     > Put the string at the bottom end of column instead of top
     _ Set width to size of length of data
+   [n] Select argument n. Use * to use the next arg as selector. 
 
  Operators:
    %% percent
@@ -62,7 +63,10 @@
    strange results can arise from things like:
       sprintf("%+080#{%s\n%}",get_dir("/"))
 
-   Ideas yet to be implemented:
+ Callbacks:
+   string _sprintf(int operator, mapping modifiers);
+
+ Ideas yet to be implemented:
    Line-break with fill? Lower-case? Upper case? Capitalize?
    Replace? Justify on decimal point?
    Change european/american decimal notation?
@@ -70,7 +74,7 @@
    Print-optimize? (space to tab, strip trailing spaces)
    '>' Kill this field in all lines but the first one
 
-   examples:
+ Examples:
 
    A short 'people' function (without sort)
 
@@ -90,7 +94,7 @@
    debug-dump
    sprintf("%78*O",width,foo);
 
-   A newspape
+   A newspaper
    sprintf("%-=*s %-=*s\n",width/2,article1,width/2,article2);
 
    A 'dotted-line' pricelist row 
@@ -99,8 +103,8 @@
 */
 
 #include "global.h"
-RCSID("$Id: sprintf.c,v 1.31 1998/11/22 11:08:52 hubbe Exp $");
-#include "error.h"
+RCSID("$Id: sprintf.c,v 1.78 2003/10/16 16:44:10 grubba Exp $");
+#include "pike_error.h"
 #include "array.h"
 #include "svalue.h"
 #include "stralloc.h"
@@ -110,6 +114,12 @@ RCSID("$Id: sprintf.c,v 1.31 1998/11/22 11:08:52 hubbe Exp $");
 #include "interpret.h"
 #include "pike_memory.h"
 #include "pike_macros.h"
+#include "object.h"
+#include "bignum.h"
+#include "mapping.h"
+#include "builtin_functions.h"
+#include "operators.h"
+#include "opcodes.h"
 #include <ctype.h>
 
 #ifdef PC
@@ -127,24 +137,34 @@ RCSID("$Id: sprintf.c,v 1.31 1998/11/22 11:08:52 hubbe Exp $");
 #define FORMAT_INFO_STACK_SIZE 200
 #define RETURN_SHARED_STRING
 
+#define SPRINTF_UNDECIDED -1027
+
 struct format_info
 {
   char *fi_free_string;
+  struct pike_string *to_free_string;
   PCHARP b;
-  int len;
-  int width;
+  ptrdiff_t len;
+  ptrdiff_t width;
   int precision;
   PCHARP pad_string;
-  int pad_length;
+  ptrdiff_t pad_length;
   int column_entries;
   short flags;
   char pos_pad;
   int column_width;
-  int column_modulo;
+  ptrdiff_t column_modulo;
 };
 
-static struct format_info format_info_stack[FORMAT_INFO_STACK_SIZE];
-static struct format_info *fsp;
+/* FIXME:
+ * re-allocate the format stack if it's too small /Hubbe
+ */
+
+struct format_stack
+{
+  struct format_info *fsp;
+  struct format_info format_info_stack[FORMAT_INFO_STACK_SIZE];
+};
 
 #define MINIMUM(X,Y) ((X)<(Y)?(X):(Y))
 
@@ -346,90 +366,104 @@ INLINE static void low_write_IEEE_float(char *b, double d, int sz)
 
 INLINE static void fix_field(struct string_builder *r,
 			     PCHARP b,
-			     int len,
+			     ptrdiff_t len,
 			     int flags,
-			     int width,
+			     ptrdiff_t width,
 			     PCHARP pad_string,
-			     int pad_length,
+			     ptrdiff_t pad_length,
 			     char pos_pad)
 {
-  int e,d;
-  if(!width)
+  ptrdiff_t e;
+  if(!width || width==SPRINTF_UNDECIDED)
   {
     if(pos_pad && EXTRACT_PCHARP(b)!='-') string_builder_putchar(r,pos_pad);
     string_builder_append(r,b,len);
     return;
   }
 
-  d=0;
   if(!(flags & DO_TRUNC) && len+(pos_pad && EXTRACT_PCHARP(b)!='-')>=width)
   {
     if(pos_pad && EXTRACT_PCHARP(b)!='-') string_builder_putchar(r,pos_pad);
     string_builder_append(r,b,len);
     return;
   }
-  if(flags & ZERO_PAD)		/* zero pad is kind of special... */
-  {
-    if(EXTRACT_PCHARP(b)=='-')
+
+  if (flags & (ZERO_PAD|FIELD_CENTER|FIELD_LEFT)) {
+    /* Some flag is set. */
+
+    if(flags & ZERO_PAD)		/* zero pad is kind of special... */
     {
-      string_builder_putchar(r,'-');
-      INC_PCHARP(b,1);
-      len--;
-      width--;
-    }else{
-      if(pos_pad)
+      if(EXTRACT_PCHARP(b)=='-')
       {
-        string_builder_putchar(r,pos_pad);
-        width--;
+  	string_builder_putchar(r,'-');
+  	INC_PCHARP(b,1);
+  	len--;
+  	width--;
+      }else{
+  	if(pos_pad)
+  	{
+  	  string_builder_putchar(r,pos_pad);
+  	  width--;
+  	}
       }
-    }
 #if 1
-    string_builder_fill(r,width-len,MKPCHARP("0",0),1,0);
+      string_builder_fill(r,width-len,MKPCHARP("0",0),1,0);
 #else
-    for(;width>len;width--) string_builder_putchar(r,'0');
+      for(;width>len;width--) string_builder_putchar(r,'0');
 #endif
-    string_builder_append(r,b,len);
-    return;
-  }
-
-  if(flags & FIELD_CENTER)
-  {
-    e=len;
-    if(pos_pad && EXTRACT_PCHARP(b)!='-') e++;
-    e=(width-e)/2;
-    if(e>0)
-    {
-      string_builder_fill(r,e,pad_string, pad_length, 0);
-      width-=e;
+      string_builder_append(r,b,len);
+      return;
     }
-    flags|=FIELD_LEFT;
-  }
+  
+    if (flags & (FIELD_CENTER|FIELD_LEFT)) {
+      ptrdiff_t d=0;
+  
+      if(flags & FIELD_CENTER)
+      {
+  	e=len;
+  	if(pos_pad && EXTRACT_PCHARP(b)!='-') e++;
+  	e=(width-e)/2;
+  	if(e>0)
+  	{
+  	  string_builder_fill(r, e, pad_string, pad_length, 0);
+  	  width-=e;
+  	}
+      }
+  
+      /* Left adjust */
+      if(pos_pad && EXTRACT_PCHARP(b)!='-')
+      {
+  	string_builder_putchar(r,pos_pad);
+  	width--;
+  	d++;
+      }
 
-  if(flags & FIELD_LEFT)
-  {
-    if(pos_pad && EXTRACT_PCHARP(b)!='-')
-    {
-      string_builder_putchar(r,pos_pad);
-      width--;
-      d++;
+#if 1
+      len = MINIMUM(width, len);
+      if (len) {
+        d += len;
+	string_builder_append(r, b, len);
+	width -= len;
+      }
+#else /* 0 */  
+      d+=MINIMUM(width,len);
+      while(len && width)
+      {
+  	string_builder_putchar(r,EXTRACT_PCHARP(b));
+  	INC_PCHARP(b,1);
+  	len--;
+  	width--;
+      }
+#endif /* 1 */
+  
+      if(width>0)
+      {
+  	d%=pad_length;
+  	string_builder_fill(r,width,pad_string,pad_length,d);
+      }
+      
+      return;
     }
-
-    d+=MINIMUM(width,len);
-    while(len && width)
-    {
-      string_builder_putchar(r,EXTRACT_PCHARP(b));
-      INC_PCHARP(b,1);
-      len--;
-      width--;
-    }
-
-    if(width>0)
-    {
-      d%=pad_length;
-      string_builder_fill(r,width,pad_string,pad_length,d);
-    }
-    
-    return;
   }
 
   /* Right-justification */
@@ -452,27 +486,26 @@ INLINE static void fix_field(struct string_builder *r,
   string_builder_append(r,b,width);
 }
 
-static struct svalue temp_svalue = { T_INT };
-
-static void free_sprintf_strings(void)
+static void free_sprintf_strings(struct format_stack *fs)
 {
-  free_svalue(&temp_svalue);
-  temp_svalue.type=T_INT;
-  for(;fsp>=format_info_stack;fsp--)
+  for(;fs->fsp>=fs->format_info_stack;fs->fsp--)
   {
-    if(fsp->fi_free_string)
-      free(fsp->fi_free_string);
-    fsp->fi_free_string=0;
+    if(fs->fsp->fi_free_string) free(fs->fsp->fi_free_string);
+    fs->fsp->fi_free_string=0;
+    if(fs->fsp->to_free_string) free_string(fs->fsp->to_free_string);
+    fs->fsp->to_free_string=0;
   }
 }
 
-static void sprintf_error(char *s,...) ATTRIBUTE((noreturn,format (printf, 1, 2)));
-static void sprintf_error(char *s,...)
+static void sprintf_error(struct format_stack *fs,
+			  char *s,...) ATTRIBUTE((noreturn,format (printf, 2, 3)));
+static void sprintf_error(struct format_stack *fs,
+			  char *s,...)
 {
   char buf[100];
   va_list args;
   va_start(args,s);
-  free_sprintf_strings();
+  free_sprintf_strings(fs);
 
   sprintf(buf,"Sprintf: %s",s);
   va_error(buf,args);
@@ -484,17 +517,18 @@ static void sprintf_error(char *s,...)
  * if there is more for next line.
  */
 
-INLINE static int do_one(struct string_builder *r,
+INLINE static int do_one(struct format_stack *fs,
+			 struct string_builder *r,
 			 struct format_info *f)
 {
   PCHARP rest;
-  int e,d,lastspace;
+  ptrdiff_t e, d, lastspace;
 
   rest.ptr=0;
   if(f->flags & (LINEBREAK|ROUGH_LINEBREAK))
   {
-    if(!f->width)
-      sprintf_error("Must have field width for linebreak.\n");
+    if(f->width==SPRINTF_UNDECIDED)
+      sprintf_error(fs, "Must have field width for linebreak.\n");
     lastspace=-1;
     for(e=0;e<f->len && e<=f->width;e++)
     {
@@ -535,8 +569,8 @@ INLINE static int do_one(struct string_builder *r,
   }
   else if(f->flags & INVERSE_COLUMN_MODE)
   {
-    if(!f->width)
-      sprintf_error("Must have field width for column mode.\n");
+    if(f->width==SPRINTF_UNDECIDED)
+      sprintf_error(fs, "Must have field width for column mode.\n");
     e=f->width/(f->column_width+1);
     if(!f->column_width || e<1) e=1;
 
@@ -568,11 +602,12 @@ INLINE static int do_one(struct string_builder *r,
   }
   else if(f->flags & COLUMN_MODE)
   {
-    int mod,col;
+    ptrdiff_t mod;
+    ptrdiff_t col;
     PCHARP end;
 
-    if(!f->width)
-      sprintf_error("Must have field width for column mode.\n");
+    if(f->width==SPRINTF_UNDECIDED)
+      sprintf_error(fs, "Must have field width for column mode.\n");
     mod=f->column_modulo;
     col=f->width/(f->column_width+1);
     if(!f->column_width || col<1) col=1;
@@ -623,7 +658,8 @@ INLINE static int do_one(struct string_builder *r,
   }
   else
   {
-    fix_field(r,f->b,f->len,f->flags,f->width,f->pad_string,f->pad_length,f->pos_pad);
+    fix_field(r,f->b,f->len,f->flags,f->width,
+	      f->pad_string,f->pad_length,f->pos_pad);
   }
 
   if(f->flags & REPEAT) return 0;
@@ -647,19 +683,32 @@ INLINE static int do_one(struct string_builder *r,
   }else{ \
     if(argument >= num_arg) \
     { \
-      sprintf_error("Too few arguments to sprintf.\n"); \
+      sprintf_error(fs, "Too few arguments to sprintf.\n"); \
       break; /* make gcc happy */ \
     } \
     VAR=lastarg=argp+(argument++); \
   }
 
+#define PEEK_SVALUE(VAR) \
+  if(arg) \
+  { \
+    VAR=arg; \
+  }else{ \
+    if(argument >= num_arg) \
+    { \
+      sprintf_error(fs, "Too few arguments to sprintf.\n"); \
+      break; /* make gcc happy */ \
+    } \
+    VAR=argp+argument; \
+  }
+
 #define GET(VAR,PIKE_TYPE,TYPE_NAME,EXTENSION) \
   { \
     struct svalue *tmp_; \
     GET_SVALUE(tmp_); \
     if(tmp_->type!=PIKE_TYPE) \
     { \
-      sprintf_error("Wrong type for argument %d: expected %s, got %s.\n",argument+1,TYPE_NAME, \
+      sprintf_error(fs, "Wrong type for argument %d: expected %s, got %s.\n",argument+1,TYPE_NAME, \
 	get_name_of_type(tmp_->type)); \
       break; /* make gcc happy */ \
     } \
@@ -672,83 +721,144 @@ INLINE static int do_one(struct string_builder *r,
 #define GET_ARRAY(VAR) GET(VAR,T_ARRAY,"array",array)
 #define GET_OBJECT(VAR) GET(VAR,T_OBJECT,"object",object)
 
-#define DO_OP() \
-   if(fsp->flags & SNURKEL) \
-   { \
-     ONERROR _e; \
-     struct array *_v; \
-     struct string_builder _b; \
-     init_string_builder(&_b,0); \
-     SET_ONERROR(_e, free_string_builder, &_b); \
-     GET_ARRAY(_v); \
-     for(tmp=0;tmp<_v->size;tmp++) \
-     { \
-       struct svalue *save_sp=sp; \
-       array_index_no_free(sp,_v,tmp); \
-       sp++; \
-       low_pike_sprintf(&_b,begin,SUBTRACT_PCHARP(a,begin)+1,sp-1,1,nosnurkel+1); \
-       if(save_sp < sp) pop_stack(); \
-     } \
-     fsp->b=MKPCHARP_STR(_b.s); \
-     fsp->len=_b.s->len; \
-     fsp->fi_free_string=(char *)_b.s; \
-     fsp->pad_string=MKPCHARP(" ",0); \
-     fsp->pad_length=1; \
-     fsp->column_width=0; \
-     fsp->pos_pad=fsp->flags=fsp->width=fsp->precision=0; \
-     UNSET_ONERROR(_e); \
-     break; \
+#define DO_OP()								\
+   if(fs->fsp->flags & SNURKEL)						\
+   {									\
+     ONERROR _e;							\
+     struct array *_v;							\
+     struct string_builder _b;						\
+     init_string_builder(&_b,0);					\
+     SET_ONERROR(_e, free_string_builder, &_b);				\
+     GET_ARRAY(_v);							\
+     for(tmp=0;tmp<_v->size;tmp++)					\
+     {									\
+       struct svalue *save_sp=sp;					\
+       array_index_no_free(sp,_v,tmp);					\
+       sp++;								\
+       low_pike_sprintf(fs, &_b,begin,SUBTRACT_PCHARP(a,begin)+1,	\
+			sp-1,1,nosnurkel+1);				\
+       if(save_sp < sp) pop_stack();					\
+     }									\
+     fs->fsp->b=MKPCHARP_STR(_b.s);					\
+     fs->fsp->len=_b.s->len;						\
+     fs->fsp->fi_free_string=(char *)_b.s;				\
+     fs->fsp->pad_string=MKPCHARP(" ",0);				\
+     fs->fsp->pad_length=1;						\
+     fs->fsp->column_width=0;						\
+     fs->fsp->pos_pad=fs->fsp->flags=0;					\
+     fs->fsp->width=fs->fsp->precision=SPRINTF_UNDECIDED;		\
+     UNSET_ONERROR(_e);							\
+     break;                                                             \
    }
 
+#define CHECK_OBJECT_SPRINTF()						      \
+	{								      \
+	   /* NOTE: It would be nice if this was a do { } while(0) macro */   \
+	   /* but it cannot be since we need to break out of the case... */   \
+	  struct svalue *sv;						      \
+	  PEEK_SVALUE(sv);						      \
+	  if(sv->type == T_OBJECT && sv->u.object->prog)		      \
+	  {                                                                   \
+            ptrdiff_t fun=FIND_LFUN(sv->u.object->prog, LFUN__SPRINTF);	      \
+	    if (fun != -1)                                                    \
+            {                                                                 \
+              int n=0;							      \
+	      push_int(EXTRACT_PCHARP(a));				      \
+	      if (fs->fsp->precision!=SPRINTF_UNDECIDED)		      \
+	      {								      \
+		 push_constant_text("precision");			      \
+		 push_int(fs->fsp->precision);				      \
+                 n+=2;							      \
+	      }								      \
+	      if (fs->fsp->width!=SPRINTF_UNDECIDED)			      \
+	      {								      \
+		 push_constant_text("width");	           		      \
+		 push_int64(fs->fsp->width);				      \
+                 n+=2;							      \
+	      }								      \
+	      if ((fs->fsp->flags&FIELD_LEFT))				      \
+	      {								      \
+		 push_constant_text("flag_left");	       		      \
+		 push_int(1);						      \
+                 n+=2;							      \
+	      }								      \
+	      f_aggregate_mapping(n);					      \
+									      \
+	      apply_low(sv->u.object, fun, 2);                                \
+	      if(sp[-1].type == T_STRING)				      \
+	      {	                                                              \
+                DO_IF_DEBUG( if(fs->fsp->to_free_string)                      \
+                             fatal("OOps in sprintf\n"); )                    \
+                fs->fsp->to_free_string = (--sp)->u.string;	              \
+									      \
+		fs->fsp->b = MKPCHARP_STR(fs->fsp->to_free_string);           \
+		fs->fsp->len = fs->fsp->to_free_string->len;		      \
+	       								      \
+                /* We have to lift one argument from the format stack. */     \
+                GET_SVALUE(sv);                                               \
+		break;							      \
+	      }								      \
+              if(!IS_ZERO(sp-1))					      \
+	      {								      \
+		 sprintf_error(fs,"argument %d (object) returned "	      \
+			       "illegal value from _sprintf()\n",argument+1); \
+	      }								      \
+	      pop_stack();						      \
+	    }								      \
+	  }								      \
+	}
 
 /* This is the main pike_sprintf function, note that it calls itself
  * recursively during the '%{ %}' parsing. The string is stored in
  * the buffer in save_objectII.c
  */
 
-static void low_pike_sprintf(struct string_builder *r,
+static void low_pike_sprintf(struct format_stack *fs,
+			     struct string_builder *r,
 			     PCHARP format,
-			     int format_len,
+			     ptrdiff_t format_len,
 			     struct svalue *argp,
-			     int num_arg,
+			     ptrdiff_t num_arg,
 			     int nosnurkel)
 {
   int argument=0;
   int tmp,setwhat,pos,d,e;
   char buffer[140];
   struct format_info *f,*start;
-  float tf;
+  double tf;
   struct svalue *arg=0;	/* pushback argument */
   struct svalue *lastarg=0;
 
   PCHARP a,begin;
   PCHARP format_end=ADD_PCHARP(format,format_len);
 
-  start=fsp;
+  start=fs->fsp;
   for(a=format;COMPARE_PCHARP(a,<,format_end);INC_PCHARP(a,1))
   {
     int num_snurkel;
 
-    fsp++;
+    if(fs->fsp-fs->format_info_stack==FORMAT_INFO_STACK_SIZE - 1)
+      sprintf_error(fs, "Sprintf stack overflow.\n");
+    fs->fsp++;
 #ifdef PIKE_DEBUG
-    if(fsp < format_info_stack)
-      fatal("sprintf: fsp out of bounds.\n");
+    if(fs->fsp < fs->format_info_stack)
+      fatal("sprintf: fs->fsp out of bounds.\n");
 #endif
-    if(fsp-format_info_stack==FORMAT_INFO_STACK_SIZE)
-      sprintf_error("Sprintf stack overflow.\n");
-    fsp->pad_string=MKPCHARP(" ",0);
-    fsp->pad_length=1;
-    fsp->fi_free_string=0;
-    fsp->column_width=0;
-    fsp->pos_pad=fsp->flags=fsp->width=fsp->precision=0;
+    fs->fsp->pad_string=MKPCHARP(" ",0);
+    fs->fsp->pad_length=1;
+    fs->fsp->fi_free_string=0;
+    fs->fsp->to_free_string=0;
+    fs->fsp->column_width=0;
+    fs->fsp->pos_pad=fs->fsp->flags=0;
+    fs->fsp->width=fs->fsp->precision=SPRINTF_UNDECIDED;
 
     if(EXTRACT_PCHARP(a)!='%')
     {
       for(e=0;INDEX_PCHARP(a,e)!='%' &&
 	    COMPARE_PCHARP(ADD_PCHARP(a,e),<,format_end);e++);
-      fsp->b=a;
-      fsp->len=e;
-      fsp->width=e;
+      fs->fsp->b=a;
+      fs->fsp->len=e;
+      fs->fsp->width=e;
       INC_PCHARP(a,e-1);
       continue;
     }
@@ -766,15 +876,20 @@ static void low_pike_sprintf(struct string_builder *r,
 	if(EXTRACT_PCHARP(a) < 256 && 
 	   isprint(EXTRACT_PCHARP(a)))
 	{
-	  sprintf_error("Error in format string, %c is not a format.\n",EXTRACT_PCHARP(a));
+	  sprintf_error(fs, "Error in format string, %c is not a format.\n",EXTRACT_PCHARP(a));
 	}else{
-	  sprintf_error("Error in format string, \\%o is not a format.\n",EXTRACT_PCHARP(a));
+	  sprintf_error(fs, "Error in format string, \\%o is not a format.\n",EXTRACT_PCHARP(a));
 	}
 	fatal("Foo, you shouldn't be here!\n");
 
         /* First the modifiers */
 
-      case '0': fsp->flags|=ZERO_PAD; continue;
+      case '0': 
+	 if (setwhat<2) 
+	 { 
+	    fs->fsp->flags|=ZERO_PAD; 
+	    continue; 
+	 }
       case '1': case '2': case '3':
       case '4': case '5': case '6':
       case '7': case '8': case '9':
@@ -786,14 +901,16 @@ static void low_pike_sprintf(struct string_builder *r,
 	GET_INT(tmp);
 
       got_arg:
-	if(tmp<1)
-	  sprintf_error("Illegal width.\n");
 	switch(setwhat)
 	{
-	case 0: fsp->width=tmp; break;
-	case 1: fsp->width=tmp;
-	case 2: fsp->precision=tmp; break;
-	case 3: fsp->column_width=tmp; break;
+	case 0:
+	case 1:
+	  if(tmp < 0) sprintf_error(fs, "Illegal width %d.\n", tmp);
+	  fs->fsp->width=tmp;
+	  if (!setwhat) break;
+	case 2: fs->fsp->precision=tmp; break;
+	case 3: fs->fsp->column_width=tmp; break;
+	case 4: fs->fsp->precision=-tmp; break;
 	}
 	continue;
 
@@ -801,22 +918,27 @@ static void low_pike_sprintf(struct string_builder *r,
       case '.': setwhat=2; continue;
       case ':': setwhat=1; continue;
 
-      case '=': fsp->flags|=LINEBREAK; continue;
-      case '/': fsp->flags|=ROUGH_LINEBREAK; continue;
-      case '#': fsp->flags|=COLUMN_MODE; continue;
-      case '$': fsp->flags|=INVERSE_COLUMN_MODE; continue;
+      case '=': fs->fsp->flags|=LINEBREAK; continue;
+      case '/': fs->fsp->flags|=ROUGH_LINEBREAK; continue;
+      case '#': fs->fsp->flags|=COLUMN_MODE; continue;
+      case '$': fs->fsp->flags|=INVERSE_COLUMN_MODE; continue;
 
-      case '-': fsp->flags|=FIELD_LEFT; continue;
-      case '|': fsp->flags|=FIELD_CENTER; continue;
-      case ' ': fsp->pos_pad=' '; continue;
-      case '+': fsp->pos_pad='+'; continue;
-      case '!': fsp->flags^=DO_TRUNC; continue;
-      case '^': fsp->flags|=REPEAT; continue;
-      case '>': fsp->flags|=MULTI_LINE_BREAK; continue;
-      case '_': fsp->flags|=WIDTH_OF_DATA; continue;
+      case '-':
+	if(setwhat==2)
+	  setwhat=4;
+	else
+	  fs->fsp->flags|=FIELD_LEFT;
+	continue;
+      case '|': fs->fsp->flags|=FIELD_CENTER; continue;
+      case ' ': fs->fsp->pos_pad=' '; continue;
+      case '+': fs->fsp->pos_pad='+'; continue;
+      case '!': fs->fsp->flags^=DO_TRUNC; continue;
+      case '^': fs->fsp->flags|=REPEAT; continue;
+      case '>': fs->fsp->flags|=MULTI_LINE_BREAK; continue;
+      case '_': fs->fsp->flags|=WIDTH_OF_DATA; continue;
       case '@':
 	if(++num_snurkel > nosnurkel)
-	  fsp->flags|=SNURKEL;
+	  fs->fsp->flags|=SNURKEL;
 	continue;
 
       case '\'':
@@ -825,12 +947,12 @@ static void low_pike_sprintf(struct string_builder *r,
 	{
 /*	  fprintf(stderr,"Sprinf-glop: %d (%c)\n",INDEX_PCHARP(a,tmp),INDEX_PCHARP(a,tmp)); */
 	  if(COMPARE_PCHARP(a,>=,format_end))
-	    sprintf_error("Unfinished pad string in format string.\n");
+	    sprintf_error(fs, "Unfinished pad string in format string.\n");
 	}
 	if(tmp)
 	{
-	  fsp->pad_string=a;
-	  fsp->pad_length=tmp;
+	  fs->fsp->pad_string=a;
+	  fs->fsp->pad_length=tmp;
 	}
 	INC_PCHARP(a,tmp);
 	continue;
@@ -839,17 +961,33 @@ static void low_pike_sprintf(struct string_builder *r,
       {
 	struct pike_string *s;
 	GET_STRING(s);
-	fsp->pad_string=MKPCHARP_STR(s);
-	fsp->pad_length=s->len;
+	if (s->len) {
+	  fs->fsp->pad_string=MKPCHARP_STR(s);
+	  fs->fsp->pad_length=s->len;
+	}
 	continue;
       }
 
       case '<':
 	if(!lastarg)
-	  sprintf_error("No last argument.\n");
+	  sprintf_error(fs, "No last argument.\n");
 	arg=lastarg;
 	continue;
 
+      case '[':
+	INC_PCHARP(a,1);
+	if(EXTRACT_PCHARP(a)=='*') {
+	  GET_INT(tmp);
+	  INC_PCHARP(a,1);
+	} else
+	  tmp=STRTOL_PCHARP(a,&a,10);
+	if(EXTRACT_PCHARP(a)!=']') 
+	  sprintf_error(fs, "Expected ] in format string, not %c.\n",EXTRACT_PCHARP(a));
+	if(tmp >= num_arg)
+	  sprintf_error(fs, "Not enough arguments to [%d].\n",tmp);
+	arg = argp+tmp;
+	continue;
+	
         /* now the real operators */
 
       case '{':
@@ -857,14 +995,14 @@ static void low_pike_sprintf(struct string_builder *r,
 	struct array *w;
 	struct string_builder b;
 #ifdef PIKE_DEBUG
-	struct format_info *fsp_save=fsp;
+	struct format_info *fsp_save=fs->fsp;
 #endif
 	DO_OP();
 	for(e=1,tmp=1;tmp;e++)
 	{
 	  if(!INDEX_PCHARP(a,e))
 	  {
-	    sprintf_error("Missing %%} in format string.\n");
+	    sprintf_error(fs, "Missing %%} in format string.\n");
 	    break;		/* UNREACHED */
 	  }
 	  if(INDEX_PCHARP(a,e)=='%')
@@ -881,8 +1019,8 @@ static void low_pike_sprintf(struct string_builder *r,
 	GET_ARRAY(w);
 	if(!w->size)
 	{
-	  fsp->b=MKPCHARP("",0);
-	  fsp->len=0;
+	  fs->fsp->b=MKPCHARP("",0);
+	  fs->fsp->len=0;
 	}else{
 	  ONERROR err;
 	  init_string_builder(&b,0);
@@ -897,24 +1035,24 @@ static void low_pike_sprintf(struct string_builder *r,
 	    s=sp;
 	    if(q)
 	    {
-	      q->array->refs++;
+	      add_ref(q->array);
 	      push_array_items(q->array);
 	    }else{
 	      array_index_no_free(sp,w,tmp);
 	      sp++;
 	    }
-	    low_pike_sprintf(&b,ADD_PCHARP(a,1),e-2,s,sp-s,0);
+	    low_pike_sprintf(fs, &b,ADD_PCHARP(a,1),e-2,s,sp-s,0);
 	    pop_n_elems(sp-s);
 	  }
 #ifdef PIKE_DEBUG
-	  if(fsp < format_info_stack)
-	    fatal("sprintf: fsp out of bounds.\n");
-	  if(fsp!=fsp_save)
-	    fatal("sprintf: fsp incorrect after recursive sprintf.\n");
+	  if(fs->fsp < fs->format_info_stack)
+	    fatal("sprintf: fs->fsp out of bounds.\n");
+	  if(fs->fsp!=fsp_save)
+	    fatal("sprintf: fs->fsp incorrect after recursive sprintf.\n");
 #endif
-	  fsp->b=MKPCHARP_STR(b.s);
-	  fsp->len=b.s->len;
-	  fsp->fi_free_string=(char *)b.s;
+	  fs->fsp->b=MKPCHARP_STR(b.s);
+	  fs->fsp->len=b.s->len;
+	  fs->fsp->fi_free_string=(char *)b.s;
 	  UNSET_ONERROR(err);
 	}
 	
@@ -923,61 +1061,154 @@ static void low_pike_sprintf(struct string_builder *r,
       }
 
       case '%':
-	fsp->b=MKPCHARP("%",0);
-	fsp->len=fsp->width=1;
+	fs->fsp->b=MKPCHARP("%",0);
+	fs->fsp->len=fs->fsp->width=1;
 	break;
 
       case 'n':
 	DO_OP();
-	fsp->b=MKPCHARP("",0);
-	fsp->len=0;
+	fs->fsp->b=MKPCHARP("",0);
+	fs->fsp->len=0;
 	break;
 
       case 't':
       {
 	struct svalue *t;
 	DO_OP();
+	CHECK_OBJECT_SPRINTF()
 	GET_SVALUE(t);
-	fsp->b=MKPCHARP(get_name_of_type(t->type),0);
-	fsp->len=strlen((char *)fsp->b.ptr);
+	fs->fsp->b=MKPCHARP(get_name_of_type(t->type),0);
+	fs->fsp->len=strlen((char *)fs->fsp->b.ptr);
 	break;
       }
 
       case 'c':
       {
-        INT32 l,tmp;
+        INT32 tmp;
+	ptrdiff_t l,n;
 	char *x;
         DO_OP();
-        l=1;
-        if(fsp->width > 0) l=fsp->width;
-	x=(char *)alloca(l);
-	fsp->b=MKPCHARP(x,0);
-	fsp->len=l;
-	GET_INT(tmp);
-        while(--l>=0)
-        {
-          x[l]=tmp & 0xff;
-          tmp>>=8;
-        }
+	CHECK_OBJECT_SPRINTF()
+	if(fs->fsp->width == SPRINTF_UNDECIDED)
+	{
+	  GET_INT(tmp);
+	  x=(char *)alloca(4);
+	  if(tmp<256) fs->fsp->b=MKPCHARP(x,0);
+	  else if(tmp<65536) fs->fsp->b=MKPCHARP(x,1);
+	  else  fs->fsp->b=MKPCHARP(x,2);
+	  SET_INDEX_PCHARP(fs->fsp->b,0,tmp);
+	  fs->fsp->len=1;
+	}
+	else if ( (fs->fsp->flags&FIELD_LEFT) )
+	{
+	  l=1;
+	  if(fs->fsp->width > 0) l=fs->fsp->width;
+	  x=(char *)alloca(l);
+	  fs->fsp->b=MKPCHARP(x,0);
+	  fs->fsp->len=l;
+	  GET_INT(tmp);
+	  n=0;
+	  while(n<l)
+	  {
+	    x[n++]=tmp & 0xff;
+	    tmp>>=8;
+	  }
+	}
+	else 
+	{
+	  l=1;
+	  if(fs->fsp->width > 0) l=fs->fsp->width;
+	  x=(char *)alloca(l);
+	  fs->fsp->b=MKPCHARP(x,0);
+	  fs->fsp->len=l;
+	  GET_INT(tmp);
+	  while(--l>=0)
+	  {
+	    x[l]=tmp & 0xff;
+	    tmp>>=8;
+	  }
+	}
 	break;
       }
 
+      /*
+	WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
+	WARNING                                                 WARNING
+	WARNING   This routine is not very well tested, so it   WARNING
+	WARNING   may give errouneous results.   /Noring        WARNING
+	WARNING                                                 WARNING
+	WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
+       */
+      case 'b':
       case 'o':
       case 'd':
       case 'u':
       case 'x':
       case 'X':
       {
-	char *x;
+	int base = 0, mask_size = 0;
+	char mode, *x;
+	INT_TYPE val;
+	
 	DO_OP();
-	GET_INT(tmp);
-	buffer[0]='%';
-	buffer[1]=EXTRACT_PCHARP(a);
-	buffer[2]=0;
-	x=(char *)alloca(100);
-	fsp->b=MKPCHARP(x,0);
-	sprintf(x,buffer,tmp);
-	fsp->len=strlen(x);
+	CHECK_OBJECT_SPRINTF()
+	GET_INT(val);
+
+	if(fs->fsp->precision != SPRINTF_UNDECIDED && fs->fsp->precision > 0)
+	  mask_size = fs->fsp->precision;
+	
+	mode=EXTRACT_PCHARP(a);
+	x=(char *)alloca(sizeof(val)*CHAR_BIT + 4 + mask_size);
+	fs->fsp->b=MKPCHARP(x,0);
+	
+	switch(mode)
+	{
+	  case 'b': base = 1; break;
+	  case 'o': base = 3; break;
+	  case 'x': base = 4; break;
+	  case 'X': base = 4; break;
+	}
+
+	if(base)
+	{
+	  char *p = x;
+	  ptrdiff_t l;
+
+	  if(mask_size || val>=0)
+	  {
+	    do {
+	      if((*p++ = '0'|(val&((1<<base)-1)))>'9')
+		p[-1] += (mode=='X'? 'A'-'9'-1 : 'a'-'9'-1);
+	      val >>= base;
+	    } while(--mask_size && val);
+	    l = p-x;
+	  }
+	  else
+	  {
+	    *p++ = '-';
+	    val = -val;
+	    do {
+	      if((*p++ = '0'|(val&((1<<base)-1)))>'9')
+		p[-1] += (mode=='X'? 'A'-'9'-1 : 'a'-'9'-1);
+	      val = ((unsigned INT_TYPE)val) >> base;
+	    } while(val);
+	    l = p-x-1;
+	  }
+	  *p = '\0';
+	  while(l>1) {
+	    char t = p[-l];
+	    p[-l] = p[-1];
+	    p[-1] = t;
+	    --p;
+	    l -= 2;
+	  }
+	}
+	else if(mode == 'u')
+	  sprintf(x, "%u", val);
+	else
+	  sprintf(x, "%d", val);
+
+	fs->fsp->len=strlen(x);
 	break;
       }
 
@@ -989,68 +1220,93 @@ static void low_pike_sprintf(struct string_builder *r,
       {
 	char *x;
 	DO_OP();
-	x=(char *)xalloc(100+MAXIMUM(fsp->width,8)+
-			      MAXIMUM(fsp->precision,3));
-	fsp->b=MKPCHARP(x,0);
+	CHECK_OBJECT_SPRINTF()
+	if (fs->fsp->precision==SPRINTF_UNDECIDED) fs->fsp->precision=3;
+
+	x=(char *)xalloc(100+MAXIMUM(fs->fsp->precision,3));
+	fs->fsp->b=MKPCHARP(x,0);
 	sprintf(buffer,"%%*.*%c",EXTRACT_PCHARP(a));
 	GET_FLOAT(tf);
-	sprintf(x,buffer,
-		fsp->width?fsp->width:8,
-		fsp->precision?fsp->precision:3,tf);
-	fsp->len=strlen(x);
-	fsp->fi_free_string=x;
+
+	if(fs->fsp->precision<0) {
+	  double m=pow(10.0, (double)fs->fsp->precision);
+	  tf = RINT(tf*m)/m;
+	} else if (fs->fsp->precision==0) {
+	  tf = RINT(tf);
+        }
+	
+	sprintf(x,buffer,1,fs->fsp->precision<0?0:fs->fsp->precision,tf);
+	fs->fsp->len=strlen(x);
+	
+	/* Make sure that the last digits really are zero. */
+	if(fs->fsp->precision<0)
+	{
+	  ptrdiff_t i, j;
+	  /* Find the ending of the number.  Yes, this can be made
+	     simpler now when the alignment bug for floats is fixed. */
+	  for(i=fs->fsp->len-1; i>=0; i--)
+ 	    if('0'<=x[i] && x[i]<='9')
+	    {
+	      i+=fs->fsp->precision+1;
+	      if(i>=0 && '0'<=x[i] && x[i]<='9')
+		for(j=0; j<-fs->fsp->precision; j++)
+		  x[i+j]='0';
+	      break;
+	    }
+	}
+
+	fs->fsp->fi_free_string=x;
 	break;
       }
 
       case 'F':
       {
-        INT32 l;
+        ptrdiff_t l;
 	char *x;
-#if defined(DOUBLE_IS_IEEE_LITTLE) || defined(DOUBLE_IS_IEEE_BIG)
-	double td;
-#endif
         DO_OP();
         l=4;
-        if(fsp->width > 0) l=fsp->width;
+        if(fs->fsp->width > 0) l=fs->fsp->width;
 	if(l != 4 && l != 8)
-	  sprintf_error("Invalid IEEE width %d.\n", l);
+	  sprintf_error(fs, "Invalid IEEE width %ld.\n",
+			PTRDIFF_T_TO_LONG(l));
 	x=(char *)alloca(l);
-	fsp->b=MKPCHARP(x,0);
-	fsp->len=l;
+	fs->fsp->b=MKPCHARP(x,0);
+	fs->fsp->len=l;
 	GET_FLOAT(tf);
 	switch(l) {
 	case 4:
+	  {
 #ifdef FLOAT_IS_IEEE_BIG
-	  MEMCPY(x, &tf, 4);
+	    float f = DO_NOT_WARN((float)tf);
+	    MEMCPY(x, &f, 4);
 #else
 #ifdef FLOAT_IS_IEEE_LITTLE
-	  x[0] = ((char *)&tf)[3];
-	  x[1] = ((char *)&tf)[2];
-	  x[2] = ((char *)&tf)[1];
-	  x[3] = ((char *)&tf)[0];
+	    float f = DO_NOT_WARN((float)tf);
+	    x[0] = ((char *)&f)[3];
+	    x[1] = ((char *)&f)[2];
+	    x[2] = ((char *)&f)[1];
+	    x[3] = ((char *)&f)[0];
 #else
-	  low_write_IEEE_float(x, (double)tf, 4);
+	    low_write_IEEE_float(x, tf, 4);
 #endif
 #endif
+	  }
 	  break;
 	case 8:
 #ifdef DOUBLE_IS_IEEE_BIG
-	  td = (double)tf;
-	  MEMCPY(x, &td, 8);
+	  MEMCPY(x, &tf, 8);
 #else
 #ifdef DOUBLE_IS_IEEE_LITTLE
-	  td = (double)tf;
-
-	  x[0] = ((char *)&td)[7];
-	  x[1] = ((char *)&td)[6];
-	  x[2] = ((char *)&td)[5];
-	  x[3] = ((char *)&td)[4];
-	  x[4] = ((char *)&td)[3];
-	  x[5] = ((char *)&td)[2];
-	  x[6] = ((char *)&td)[1];
-	  x[7] = ((char *)&td)[0];
+	  x[0] = ((char *)&tf)[7];
+	  x[1] = ((char *)&tf)[6];
+	  x[2] = ((char *)&tf)[5];
+	  x[3] = ((char *)&tf)[4];
+	  x[4] = ((char *)&tf)[3];
+	  x[5] = ((char *)&tf)[2];
+	  x[6] = ((char *)&tf)[1];
+	  x[7] = ((char *)&tf)[0];
 #else
-	  low_write_IEEE_float(x, (double)tf, 8);
+	  low_write_IEEE_float(x, tf, 8);
 #endif
 #endif
 	}
@@ -1062,13 +1318,15 @@ static void low_pike_sprintf(struct string_builder *r,
 	string s;
 	struct svalue *t;
 	DO_OP();
+	/* No need to do CHECK_OBJECT_SPRINTF() here,
+	   it is checked in describe_svalue. */
 	GET_SVALUE(t);
 	init_buf();
 	describe_svalue(t,0,0);
 	s=complex_free_buf();
-	fsp->b=MKPCHARP(s.str,0);
-	fsp->len=s.len;
-	fsp->fi_free_string=s.str;
+	fs->fsp->b=MKPCHARP(s.str,0);
+	fs->fsp->len=s.len;
+	fs->fsp->fi_free_string=s.str;
 	break;
       }
 
@@ -1076,9 +1334,12 @@ static void low_pike_sprintf(struct string_builder *r,
       {
 	struct pike_string *s;
 	DO_OP();
+	CHECK_OBJECT_SPRINTF()
 	GET_STRING(s);
-	fsp->b=MKPCHARP_STR(s);
-	fsp->len=s->len;
+	fs->fsp->b=MKPCHARP_STR(s);
+	fs->fsp->len=s->len;
+	if(fs->fsp->precision != SPRINTF_UNDECIDED && fs->fsp->precision < fs->fsp->len)
+	  fs->fsp->len = (fs->fsp->precision < 0 ? 0 : fs->fsp->precision);
 	break;
       }
       }
@@ -1086,14 +1347,16 @@ static void low_pike_sprintf(struct string_builder *r,
     }
   }
 
-  for(f=fsp;f>start;f--)
+  for(f=fs->fsp;f>start;f--)
   {
-    if(f->flags & WIDTH_OF_DATA) f->width=f->len;
+    if(f->flags & WIDTH_OF_DATA)
+      f->width=f->len;
 
     if(((f->flags & INVERSE_COLUMN_MODE) && !f->column_width) ||
        (f->flags & COLUMN_MODE))
     {
-      int max_len,nr,columns;
+      int max_len,nr;
+      ptrdiff_t columns;
       tmp=1;
       for(max_len=nr=e=0;e<f->len;e++)
       {
@@ -1106,17 +1369,21 @@ static void low_pike_sprintf(struct string_builder *r,
 	tmp++;
       }
       nr++;
-      if(max_len<tmp) max_len=tmp;
-      if(!f->column_width) f->column_width=max_len;
+      if(max_len<tmp)
+	max_len=tmp;
+      if(!f->column_width)
+	f->column_width=max_len;
       f->column_entries=nr;
       columns=f->width/(f->column_width+1);
-      if(f->column_width<1 || columns<1) columns=1;
+      
+      if(f->column_width<1 || columns<1)
+	columns=1;
       f->column_modulo=(nr+columns-1)/columns;
     }
   }
 
   /* Here we do some DWIM */
-  for(f=fsp-1;f>start;f--)
+  for(f=fs->fsp-1;f>start;f--)
   {
     if((f[1].flags & MULTILINE) &&
        !(f[0].flags & (MULTILINE|MULTI_LINE_BREAK)))
@@ -1124,28 +1391,39 @@ static void low_pike_sprintf(struct string_builder *r,
       if(! MEMCHR_PCHARP(f->b, '\n', f->len).ptr ) f->flags|=MULTI_LINE;
     }
   }
-  for(f=start+1;f<=fsp;)
+  
+  for(f=start+1;f<=fs->fsp;)
   {
-    for(;f<=fsp && !(f->flags&MULTILINE);f++) do_one(r,f);
-    do
-    {
+    for(;f<=fs->fsp && !(f->flags&MULTILINE);f++)
+      do_one(fs, r, f);
+    
+    do {
       d=0;
-      for(e=0;f+e<=fsp && (f[e].flags & MULTILINE);e++) d|=do_one(r,f+e);
-      if(d) string_builder_putchar(r,'\n');
-    }while(d);
+      for(e=0;f+e<=fs->fsp && (f[e].flags & MULTILINE);e++)
+	d |= do_one(fs, r, f+e);
+      if(d)
+	string_builder_putchar(r,'\n');
+    } while(d);
 
-    for(;f<=fsp && (f->flags&MULTILINE); f++);
+    for(;f<=fs->fsp && (f->flags&MULTILINE); f++);
   }
 
-  while(fsp>start)
+  while(fs->fsp>start)
   {
 #ifdef PIKE_DEBUG
-    if(fsp < format_info_stack)
+    if(fs->fsp < fs->format_info_stack)
       fatal("sprintf: fsp out of bounds.\n");
 #endif
-    if(fsp->fi_free_string) free(fsp->fi_free_string);
-    fsp->fi_free_string=0;
-    fsp--;
+    
+    if(fs->fsp->fi_free_string)
+      free(fs->fsp->fi_free_string);
+    fs->fsp->fi_free_string=0;
+    
+    if(fs->fsp->to_free_string)
+      free_string(fs->fsp->to_free_string);
+    fs->fsp->to_free_string=0;
+    
+    fs->fsp--;
   }
 }
 
@@ -1166,42 +1444,60 @@ string pike_sprintf(char *format,struct svalue *argp,int num_arg)
 /* The efun */
 void f_sprintf(INT32 num_arg)
 {
-  ONERROR tmp;
+  ONERROR err_string_builder, err_format_stack;
   struct pike_string *ret;
   struct svalue *argp;
   struct string_builder r;
 
-  argp=sp-num_arg;
-  free_sprintf_strings();
-  fsp=format_info_stack-1;
+  struct format_stack fs;
 
-  if(argp[0].type != T_STRING)
-    error("Bad argument 1 to sprintf.\n");
+  argp=sp-num_arg;
+  
+  fs.fsp = fs.format_info_stack-1;
+
+  if(argp[0].type != T_STRING) {
+    if (argp[0].type == T_OBJECT) {
+      /* Try checking if we can cast it to a string... */
+      ref_push_object(argp[0].u.object);
+      o_cast(string_type_string, PIKE_T_STRING);
+      if (sp[-1].type != T_STRING) {
+	/* We don't accept objects... */
+	Pike_error("sprintf(): Cast to string failed.\n");
+      }
+      /* Replace the original object with the new string. */
+      assign_svalue(argp, sp-1);
+      /* Clean up the stack. */
+      pop_stack();
+    } else {
+      Pike_error("Bad argument 1 to sprintf.\n");
+    }
+  }
 
   init_string_builder(&r,0);
-  SET_ONERROR(tmp, free_string_builder, &r);
-  low_pike_sprintf(&r,
+  SET_ONERROR(err_format_stack, free_sprintf_strings, &fs);
+  SET_ONERROR(err_string_builder, free_string_builder, &r);
+  low_pike_sprintf(&fs,
+		   &r,
 		   MKPCHARP_STR(argp->u.string),
 		   argp->u.string->len,
 		   argp+1,
 		   num_arg-1,
 		   0);
-  UNSET_ONERROR(tmp);
+  UNSET_ONERROR(err_string_builder);
+  UNSET_ONERROR(err_format_stack);
   ret=finish_string_builder(&r);
 
-  free_svalue(&temp_svalue);
-  temp_svalue.type=T_INT;
   pop_n_elems(num_arg);
   push_string(ret);
 }
 
 void pike_module_init(void)
 {
-  add_efun("sprintf", f_sprintf,"function(string, mixed ... : string)",
+  /* function(string|object, mixed ... : string) */
+  ADD_EFUN("sprintf", f_sprintf, tFuncV(tOr(tStr, tObj), tMix, tStr),
 	   OPT_TRY_OPTIMIZE);
 }
 
 void pike_module_exit(void)
 {
-  free_sprintf_strings();
 }
