@@ -1,5 +1,5 @@
 /*
- * $Id: threads.h,v 1.120 2003/12/13 20:37:53 jonasw Exp $
+ * $Id: threads.h,v 1.123 2004/05/01 12:19:47 mast Exp $
  */
 #ifndef THREADS_H
 #define THREADS_H
@@ -85,11 +85,14 @@ PMOD_EXPORT extern int live_threads;
 struct object;
 PMOD_EXPORT extern size_t thread_stack_size;
 
-PMOD_EXPORT void thread_low_error (int errcode);
+PMOD_EXPORT void thread_low_error (int errcode, const char *cmd,
+                                   const char *fname, int lineno);
 
 #define LOW_THREAD_CHECK_NONZERO_ERROR(CALL) do {			\
     int thread_errcode_ = (CALL);					\
-    if (thread_errcode_) thread_low_error (thread_errcode_);		\
+    if (thread_errcode_)                                                \
+      thread_low_error(thread_errcode_, TOSTR(CALL),                    \
+                       __FILE__, __LINE__);                             \
   } while (0)
 
 #define DEFINE_MUTEX(X) PIKE_MUTEX_T X
@@ -287,7 +290,8 @@ extern pthread_attr_t small_pattr;
 #include <windows.h>
  
 #define LOW_THREAD_CHECK_ZERO_ERROR(CALL) do {				\
-    if (!(CALL)) thread_low_error (GetLastError());			\
+    if (!(CALL)) thread_low_error (GetLastError(), TOSTR(CALL),		\
+				   __FILE__, __LINE__);			\
   } while (0)
 
 #define THREAD_T unsigned
@@ -542,11 +546,11 @@ PMOD_EXPORT extern int t_flag;
      struct thread_state *_tmp=OBJ2THREAD(Pike_interpreter.thread_id); \
      SWAP_OUT_THREAD(_tmp); \
      THREADS_FPRINTF(1, (stderr, "SWAP_OUT_CURRENT_THREAD() %s:%d t:%08x\n", \
-			 __FILE__, __LINE__, (unsigned int)_tmp->thread_id)) \
+			 __FILE__, __LINE__, (unsigned int)_tmp->id)) \
 
 #define SWAP_IN_CURRENT_THREAD() \
    THREADS_FPRINTF(1, (stderr, "SWAP_IN_CURRENT_THREAD() %s:%d ... t:%08x\n", \
-		       __FILE__, __LINE__, (unsigned int)_tmp->thread_id)); \
+		       __FILE__, __LINE__, (unsigned int)_tmp->id)); \
    SWAP_IN_THREAD(_tmp);\
  } while(0)
 
@@ -590,7 +594,7 @@ PMOD_EXPORT extern int Pike_in_gc;
        SWAP_OUT_THREAD(_tmp); \
        THREADS_FPRINTF(1, (stderr, "THREADS_ALLOW() %s:%d t:%08x(#%d)\n", \
 			   __FILE__, __LINE__, \
-			   (unsigned int)_tmp->thread_id, live_threads)); \
+			   (unsigned int)_tmp->id, live_threads)); \
        mt_unlock_interpreter(); \
      } else {} \
      HIDE_GLOBAL_VARIABLES()
@@ -601,7 +605,7 @@ PMOD_EXPORT extern int Pike_in_gc;
        mt_lock_interpreter(); \
        THREADS_FPRINTF(1, (stderr, "THREADS_DISALLOW() %s:%d t:%08x(#%d)\n", \
 			   __FILE__, __LINE__, \
-                           (unsigned int)_tmp->thread_id, live_threads)); \
+			   (unsigned int)_tmp->id, live_threads)); \
        while (threads_disabled) { \
          THREADS_FPRINTF(1, (stderr, \
                              "THREADS_DISALLOW(): Threads disabled\n")); \
@@ -631,7 +635,7 @@ PMOD_EXPORT extern int Pike_in_gc;
        live_threads++; \
        THREADS_FPRINTF(1, (stderr, "THREADS_ALLOW_UID() %s:%d t:%08x(#%d)\n", \
 			   __FILE__, __LINE__, \
-			   (unsigned int)_tmp_uid->thread_id, live_threads)); \
+			   (unsigned int)_tmp_uid->id, live_threads)); \
        mt_unlock_interpreter(); \
      } else {} \
      HIDE_GLOBAL_VARIABLES()
@@ -644,7 +648,7 @@ PMOD_EXPORT extern int Pike_in_gc;
        THREADS_FPRINTF(1, (stderr, \
                            "THREADS_DISALLOW_UID() %s:%d t:%08x(#%d)\n", \
 			   __FILE__, __LINE__, \
-                           (unsigned int)_tmp_uid->thread_id, live_threads)); \
+			   (unsigned int)_tmp_uid->id, live_threads)); \
        co_broadcast(&live_threads_change); \
        while (threads_disabled) { \
          THREADS_FPRINTF(1, (stderr, "THREADS_DISALLOW_UID(): Wait...\n")); \
